from django.contrib.auth import get_permission_codename, get_user_model
from django.contrib.auth.models import Group
from django.core.exceptions import FieldDoesNotExist, ImproperlyConfigured
from django.db.models import Q

from wagtail.models import Collection, GroupCollectionPermission

from .base import BaseDjangoAuthPermissionPolicy


class CollectionPermissionLookupMixin:
    permission_cache_name = "_collection_permission_cache"

    def _get_user_permission_objects_for_actions(self, user, actions):
        """
        Get a set of the user's GroupCollectionPermission objects for the given actions
        """
        permission_codenames = {
            get_permission_codename(action, self.auth_model._meta) for action in actions
        }
        return {
            group_permission
            for group_permission in self.get_cached_permissions_for_user(user)
            if group_permission.permission.codename in permission_codenames
        }

    def get_all_permissions_for_user(self, user):
        # For these users, we can determine the permissions without querying
        # GroupCollectionPermission by checking it directly in _check_perm()
        if not user.is_active or user.is_anonymous or user.is_superuser:
            return GroupCollectionPermission.objects.none()

        return GroupCollectionPermission.objects.filter(
            group__user=user
        ).select_related("permission", "collection")

    def _check_perm(self, user, actions, collection=None):
        """
        Equivalent to user.has_perm(self._get_permission_name(action)) on all listed actions,
        but using GroupCollectionPermission rather than group.permissions.
        If collection is specified, only consider GroupCollectionPermission records
        that apply to that collection.
        """
        if not (user.is_active and user.is_authenticated):
            return False

        if user.is_superuser:
            return True

        collection_permissions = self._get_user_permission_objects_for_actions(
            user, actions
        )

        if collection:
            collection_permissions = {
                permission
                for permission in collection_permissions
                if collection.is_descendant_of(permission.collection)
                or collection.pk == permission.collection_id
            }

        return bool(collection_permissions)

    def _collections_with_perm(self, user, actions):
        """
        Return a queryset of collections on which this user has a GroupCollectionPermission
        record for any of the given actions, either on the collection itself or an ancestor
        """
        permissions = self._get_user_permission_objects_for_actions(user, actions)

        collections = Collection.objects.none()
        for perm in permissions:
            collections |= Collection.objects.descendant_of(
                perm.collection, inclusive=True
            )

        return collections

    def _users_with_perm_filter(self, actions, collection=None):
        """
        Return a filter expression that will filter a user queryset to those with any
        permissions corresponding to 'actions', via either GroupCollectionPermission
        or superuser privileges.
        If collection is specified, only consider GroupCollectionPermission records
        that apply to that collection.
        """
        permissions = self._get_permission_objects_for_actions(actions)
        # Find all groups with GroupCollectionPermission records for
        # any of these permissions
        groups = Group.objects.filter(
            collection_permissions__permission__in=permissions,
        )

        if collection is not None:
            collections = collection.get_ancestors(inclusive=True)
            groups = groups.filter(collection_permissions__collection__in=collections)

        # Find all users who are active, and are superusers or in any of these groups
        return Q(is_active=True) & (Q(is_superuser=True) | Q(groups__in=groups))

    def _users_with_perm(self, actions, collection=None):
        """
        Return a queryset of users with any permissions corresponding to 'actions',
        via either GroupCollectionPermission or superuser privileges.
        If collection is specified, only consider GroupCollectionPermission records
        that apply to that collection.
        """
        return (
            get_user_model()
            .objects.filter(
                self._users_with_perm_filter(actions, collection=collection)
            )
            .distinct()
        )

    def collections_user_has_any_permission_for(self, user, actions):
        """
        Return a queryset of all collections in which the given user has
        permission to perform any of the given actions
        """
        if user.is_active and user.is_superuser:
            # active superusers can perform any action (including unrecognised ones)
            # in any collection
            return Collection.objects.all()

        if not user.is_authenticated:
            return Collection.objects.none()

        return self._collections_with_perm(user, actions)

    def collections_user_has_permission_for(self, user, action):
        """
        Return a queryset of all collections in which the given user has
        permission to perform the given action
        """
        return self.collections_user_has_any_permission_for(user, [action])


class CollectionPermissionPolicy(
    CollectionPermissionLookupMixin, BaseDjangoAuthPermissionPolicy
):
    """
    A permission policy for objects that are assigned locations in the Collection tree.
    Permissions may be defined at any node of the hierarchy, through the
    GroupCollectionPermission model, and propagate downwards. These permissions are
    applied to objects according to the standard django.contrib.auth permission model.
    """

    def user_has_permission(self, user, action):
        """
        Return whether the given user has permission to perform the given action
        on some or all instances of this model
        """
        return self._check_perm(user, [action])

    def user_has_any_permission(self, user, actions):
        """
        Return whether the given user has permission to perform any of the given actions
        on some or all instances of this model
        """
        return self._check_perm(user, actions)

    def users_with_any_permission(self, actions):
        """
        Return a queryset of users who have permission to perform any of the given actions
        on some or all instances of this model
        """
        return self._users_with_perm(actions)

    def user_has_permission_for_instance(self, user, action, instance):
        """
        Return whether the given user has permission to perform the given action on the
        given model instance
        """
        return self._check_perm(user, [action], collection=instance.collection)

    def user_has_any_permission_for_instance(self, user, actions, instance):
        """
        Return whether the given user has permission to perform any of the given actions
        on the given model instance
        """
        return self._check_perm(user, actions, collection=instance.collection)

    def instances_user_has_any_permission_for(self, user, actions):
        """
        Return a queryset of all instances of this model for which the given user has
        permission to perform any of the given actions
        """
        if not (user.is_active and user.is_authenticated):
            return self.model._default_manager.none()
        elif user.is_superuser:
            return self.model._default_manager.all()
        else:
            # filter to just the collections with this permission
            return self.model._default_manager.filter(
                collection__in=list(self._collections_with_perm(user, actions))
            )

    def users_with_any_permission_for_instance(self, actions, instance):
        """
        Return a queryset of all users who have permission to perform any of the given
        actions on the given model instance
        """
        return self._users_with_perm(actions, collection=instance.collection)


class CollectionOwnershipPermissionPolicy(
    CollectionPermissionLookupMixin, BaseDjangoAuthPermissionPolicy
):
    """
    A permission policy for objects that are assigned locations in the Collection tree.
    Permissions may be defined at any node of the hierarchy, through the
    GroupCollectionPermission model, and propagate downwards. These permissions are
    applied to objects according to the 'ownership' permission model
    (see permission_policies.base.OwnershipPermissionPolicy)
    """

    def __init__(self, model, auth_model=None, owner_field_name="owner"):
        super().__init__(model, auth_model=auth_model)
        self.owner_field_name = owner_field_name

    def check_model(self, model):
        super().check_model(model)

        # make sure owner_field_name is a field that exists on the model
        try:
            model._meta.get_field(self.owner_field_name)
        except FieldDoesNotExist:
            raise ImproperlyConfigured(
                "%s has no field named '%s'. To use this model with "
                "CollectionOwnershipPermissionPolicy, you must specify a valid field name as "
                "owner_field_name." % (model, self.owner_field_name)
            )

    def user_has_permission(self, user, action):
        if action == "add":
            return self._check_perm(user, ["add"])
        elif action == "choose":
            return self._check_perm(user, ["choose"])
        elif action == "change" or action == "delete":
            # having 'add' permission means that there are *potentially*
            # some instances they can edit (namely: ones they own),
            # which is sufficient for returning True here
            return self._check_perm(user, ["add", "change"])
        else:
            # unrecognised actions are only allowed for active superusers
            return user.is_active and user.is_superuser

    def users_with_any_permission(self, actions):
        known_actions = set(actions) & {"add", "choose", "change"}

        # "delete" is considered equivalent to "change"
        if "delete" in actions:
            known_actions.add("change")

        # users with only "add" permission can still change instances they own
        if "change" in known_actions:
            known_actions.add("add")

        if not known_actions:
            # none of the actions passed in here are ones that we recognise, so only
            # allow them for active superusers
            return get_user_model().objects.filter(is_active=True, is_superuser=True)

        return self._users_with_perm(known_actions)

    def user_has_permission_for_instance(self, user, action, instance):
        return self.user_has_any_permission_for_instance(user, [action], instance)

    def user_has_any_permission_for_instance(self, user, actions, instance):
        known_actions = set(actions) & {"add", "choose", "change"}

        # "delete" is considered equivalent to "change"
        if "delete" in actions:
            known_actions.add("change")

        # users with only "add" permission can still change instances they own
        if (
            "change" in known_actions
            and getattr(instance, self.owner_field_name) == user
        ):
            known_actions.add("add")

        if known_actions:
            return self._check_perm(user, known_actions, collection=instance.collection)
        else:
            # 'change', 'delete', and 'choose' are the only actions that are well-defined
            # for specific instances. Other actions are only available to
            # active superusers.
            return user.is_active and user.is_superuser

    def instances_user_has_any_permission_for(self, user, actions):
        known_actions = set(actions) & {"change", "choose"}

        # "delete" is considered equivalent to "change"
        if "delete" in actions:
            known_actions.add("change")

        if user.is_active and user.is_superuser:
            # active superusers can perform any action (including unrecognised ones)
            # on any instance
            return self.model._default_manager.all()
        elif not user.is_authenticated:
            return self.model._default_manager.none()
        elif known_actions:
            # if "change" or "delete" in actions, return instances which are:
            #   - in (a descendant of) a collection for which they have "change" permission
            #   - OR in (a descendant of) a collection for which they have "add" permission,
            #     and are owned by them
            # if "choose" in actions, return instances which are:
            #   - in (a descendant of) a collection for which they have "choose" permission.
            # Note that if the actions contain both cases, the results will be combined
            # because the user has "any" of the permissions in the actions.

            collections = self._collections_with_perm(user, known_actions)
            perm_filter = Q(collection__in=collections)

            # "add" permission implies "change" permission,
            # but only if the instance is owned by the user
            if "change" in known_actions:
                perm_filter |= Q(
                    collection__in=self._collections_with_perm(user, ["add"])
                ) & Q(**{self.owner_field_name: user})

            return self.model._default_manager.filter(perm_filter)
        else:
            # action is either not recognised, or is the 'add' action which is
            # not meaningful for existing instances. As such, non-superusers
            # cannot perform it on any existing instances.
            return self.model._default_manager.none()

    def users_with_any_permission_for_instance(self, actions, instance):
        known_actions = set(actions) & {"choose", "change"}

        # "delete" is considered equivalent to "change"
        if "delete" in actions:
            known_actions.add("change")

        filter_expr = self._users_with_perm_filter(
            known_actions, collection=instance.collection
        )

        # users with only "add" permission can still change instances they own
        if "change" in known_actions:
            owner = getattr(instance, self.owner_field_name)
            if owner is not None and self._check_perm(
                owner, {"add"}, collection=instance.collection
            ):
                filter_expr |= Q(pk=owner.pk)

        if known_actions:
            return get_user_model().objects.filter(filter_expr).distinct()
        else:
            # action is either not recognised, or is the 'add' action which is
            # not meaningful for existing instances. As such, the action is only
            # available to superusers
            return get_user_model().objects.filter(is_active=True, is_superuser=True)

    def collections_user_has_any_permission_for(self, user, actions):
        """
        Return a queryset of all collections in which the given user has
        permission to perform any of the given actions
        """
        known_actions = set(actions) & {"add", "choose", "change"}

        # "delete" is considered equivalent to "change"
        if "delete" in actions:
            known_actions.add("change")

        # users with only "add" permission can still change instances they own
        if "change" in known_actions:
            known_actions.add("add")

        if user.is_active and user.is_superuser:
            # active superusers can perform any action (including unrecognised ones)
            # in any collection
            return Collection.objects.all()

        elif not user.is_authenticated:
            return Collection.objects.none()

        elif known_actions:
            return self._collections_with_perm(user, known_actions)

        else:
            # action is not recognised, and so non-superusers
            # cannot perform it on any existing collections
            return Collection.objects.none()


class CollectionManagementPermissionPolicy(
    CollectionPermissionLookupMixin, BaseDjangoAuthPermissionPolicy
):
    def _descendants_with_perm(self, user, action):
        """
        Return a queryset of collections descended from a collection on which this user has
        a GroupCollectionPermission record for this action. Used for actions, like edit and
        delete where the user cannot modify the collection where they are granted permission.
        """
        # Get the permission object corresponding to this action
        permission = self._get_permission_objects_for_actions([action]).first()

        # Get the collections that have a GroupCollectionPermission record
        # for this permission and any of the user's groups;
        # create a list of their paths
        collection_roots = Collection.objects.filter(
            group_permissions__group__in=user.groups.all(),
            group_permissions__permission=permission,
        ).values("path", "depth")

        if collection_roots:
            # build a filter expression that will filter our model to just those
            # instances in collections with a path that starts with one of the above
            # but excluding the collection on which permission was granted
            collection_path_filter = Q(
                path__startswith=collection_roots[0]["path"]
            ) & Q(depth__gt=collection_roots[0]["depth"])
            for collection in collection_roots[1:]:
                collection_path_filter = collection_path_filter | (
                    Q(path__startswith=collection["path"])
                    & Q(depth__gt=collection["depth"])
                )
            return Collection.objects.all().filter(collection_path_filter)
        else:
            # no matching collections
            return Collection.objects.none()

    def user_has_permission(self, user, action):
        """
        Return whether the given user has permission to perform the given action
        on some or all instances of this model
        """
        return self.user_has_any_permission(user, [action])

    def user_has_any_permission(self, user, actions):
        """
        Return whether the given user has permission to perform any of the given actions
        on some or all instances of this model.
        """
        return self._check_perm(user, actions)

    def users_with_any_permission(self, actions):
        """
        Return a queryset of users who have permission to perform any of the given actions
        on some or all instances of this model
        """
        return self._users_with_perm(actions)

    def user_has_permission_for_instance(self, user, action, instance):
        """
        Return whether the given user has permission to perform the given action on the
        given model instance
        """
        return self._check_perm(user, [action], collection=instance)

    def user_has_any_permission_for_instance(self, user, actions, instance):
        """
        Return whether the given user has permission to perform any of the given actions
        on the given model instance
        """
        return self._check_perm(user, actions, collection=instance)

    def users_with_any_permission_for_instance(self, actions, instance):
        """
        Return a queryset of all users who have permission to perform any of the given
        actions on the given model instance
        """
        return self._users_with_perm(actions, collection=instance)

    def instances_user_has_permission_for(self, user, action):
        if user.is_active and user.is_superuser:
            # active superusers can perform any action (including unrecognised ones)
            # in any collection - except for deleting the root collection
            if action == "delete":
                return Collection.objects.exclude(depth=1).all()
            else:
                return Collection.objects.all()

        elif not user.is_authenticated:
            return Collection.objects.none()

        else:
            if action == "delete":
                return self._descendants_with_perm(user, action)
            else:
                return self._collections_with_perm(user, [action])

    def instances_user_has_any_permission_for(self, user, actions):
        return self.collections_user_has_any_permission_for(user, actions)



from django import VERSION as DJANGO_VERSION
from django import forms
from django.conf import settings
from django.core.exceptions import ImproperlyConfigured

# A setting that can be used in foreign key declarations
AUTH_USER_MODEL = getattr(settings, "AUTH_USER_MODEL", "auth.User")
# Two additional settings that are useful in South migrations when
# specifying the user model in the FakeORM
try:
    AUTH_USER_APP_LABEL, AUTH_USER_MODEL_NAME = AUTH_USER_MODEL.rsplit(".", 1)
except ValueError:
    raise ImproperlyConfigured(
        "AUTH_USER_MODEL must be of the form 'app_label.model_name'"
    )


try:
    from http import HTTPMethod
except ImportError:
    # For Python < 3.11
    from enum import Enum

    class HTTPMethod(Enum):
        GET = "GET"
        HEAD = "HEAD"
        OPTIONS = "OPTIONS"
        POST = "POST"
        PUT = "PUT"
        DELETE = "DELETE"
        PATCH = "PATCH"


URLField = forms.URLField

# Prevent deprecation warning about the default scheme changing from "http" to
# "https" in Django 5.0, while also avoiding the need to set the
# FORMS_URLFIELD_ASSUME_HTTPS that would raise a different deprecation warning.
# Remove the following block when the minimum Django version is >= 5.0.
if (5, 0) <= DJANGO_VERSION < (6, 0):

    class URLField(forms.URLField):
        def __init__(self, *args, **kwargs):
            kwargs.setdefault("assume_scheme", "https")
            super().__init__(*args, **kwargs)



from django.utils.safestring import mark_safe
from django.utils.translation import gettext

from wagtail.admin.ui.tables import BaseColumn, BulkActionsCheckboxColumn, Column, Table


class PageTitleColumn(BaseColumn):
    header_template_name = "wagtailadmin/pages/listing/_page_title_column_header.html"
    cell_template_name = "wagtailadmin/pages/listing/_page_title_cell.html"
    classname = "title"

    def get_header_context_data(self, parent_context):
        context = super().get_header_context_data(parent_context)
        parent_page = parent_context.get("parent_page")

        context["items_count"] = parent_context.get("items_count")
        context["page_obj"] = parent_context.get("page_obj")
        context["parent_page"] = parent_page

        if parent_page and (
            parent_context.get("is_searching") or parent_context.get("is_filtering")
        ):
            # Results are switchable between searching the whole tree and searching just this parent.
            # Add extra signposting to show which scope we're in, and provide a link to switch scope.
            if parent_context.get("is_searching_whole_tree"):
                context["result_scope"] = "whole_tree"
            else:
                context["result_scope"] = "parent"
        else:
            # No signposting needed
            context["result_scope"] = None

        # If results are not paginated e.g. when using the OrderingColumn,
        # all items are displayed on the page
        context["start_index"] = 1
        context["end_index"] = context["items_count"]
        if context["page_obj"]:
            context["start_index"] = context["page_obj"].start_index()
            context["end_index"] = context["page_obj"].end_index()

        return context

    def get_cell_context_data(self, instance, parent_context):
        context = super().get_cell_context_data(instance, parent_context)
        context["page_perms"] = instance.permissions_for_user(
            parent_context["request"].user
        )
        context["parent_page"] = getattr(instance, "annotated_parent_page", None)
        context["show_locale_labels"] = parent_context.get("show_locale_labels")
        context["perms"] = parent_context.get("perms")
        context["actions_next_url"] = parent_context.get("actions_next_url")
        return context


class ParentPageColumn(Column):
    cell_template_name = "wagtailadmin/pages/listing/_parent_page_cell.html"

    def get_value(self, instance):
        if parent := getattr(instance, "_parent_page", None):
            return parent
        return instance.get_parent()


class PageStatusColumn(BaseColumn):
    cell_template_name = "wagtailadmin/pages/listing/_page_status_cell.html"


class BulkActionsColumn(BulkActionsCheckboxColumn):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs, obj_type="page")

    def get_header_context_data(self, parent_context):
        context = super().get_header_context_data(parent_context)
        parent_page = parent_context.get("parent_page")
        if parent_page:
            context["parent"] = parent_page.id
        return context


class OrderingColumn(BaseColumn):
    header_template_name = "wagtailadmin/pages/listing/_ordering_header.html"
    cell_template_name = "wagtailadmin/pages/listing/_ordering_cell.html"


class PageTypeColumn(Column):
    def get_header_context_data(self, parent_context):
        context = super().get_header_context_data(parent_context)
        # Cannot order by page type while searching, due to
        # https://github.com/wagtail/wagtail/issues/6616
        context["is_orderable"] = not parent_context.get("is_searching")
        return context


class NavigateToChildrenColumn(BaseColumn):
    cell_template_name = "wagtailadmin/pages/listing/_navigation_explore.html"

    def get_cell_context_data(self, instance, parent_context):
        context = super().get_cell_context_data(instance, parent_context)
        context["page"] = instance
        context["page_perms"] = instance.permissions_for_user(
            parent_context["request"].user
        )
        return context

    def render_header_html(self, parent_context):
        # This column has no header, as the cell's function will vary between "explore child pages"
        # and "add child page", and this link provides all the signposting needed. Render it as a
        # <td> rather than <th> as headings cannot be empty (https://dequeuniversity.com/rules/axe/4.9/empty-table-header).
        return mark_safe("<td></td>")


class PageTable(Table):
    def __init__(
        self,
        *args,
        use_row_ordering_attributes=False,
        parent_page=None,
        show_locale_labels=False,
        actions_next_url=None,
        **kwargs,
    ):
        super().__init__(*args, **kwargs)

        # If true, attributes will be added on the <tr> element to support reordering
        self.use_row_ordering_attributes = use_row_ordering_attributes

        # The parent page of the pages being listed - used to add extra context to the title text
        # of the reordering links. Leave this undefined if the pages being listed do not share a
        # common parent.
        self.parent_page = parent_page
        if self.parent_page:
            # Use more detailed title text that mentions the parent page, if we have one and the
            # current strings have not been overridden from Table's default
            if self.ascending_title_text_format == Table.ascending_title_text_format:
                self.ascending_title_text_format = gettext(
                    "Sort the order of child pages within '%(parent)s' by '%(label)s' in ascending order."
                )
            if self.descending_title_text_format == Table.descending_title_text_format:
                self.descending_title_text_format = gettext(
                    "Sort the order of child pages within '%(parent)s' by '%(label)s' in descending order."
                )

        self.show_locale_labels = show_locale_labels
        self.actions_next_url = actions_next_url

    def get_ascending_title_text(self, column):
        return self.ascending_title_text_format % {
            "parent": self.parent_page and self.parent_page.get_admin_display_title(),
            "label": column.label,
        }

    def get_descending_title_text(self, column):
        return self.descending_title_text_format % {
            "parent": self.parent_page and self.parent_page.get_admin_display_title(),
            "label": column.label,
        }

    def get_row_classname(self, instance):
        if not instance.live:
            return "unpublished"
        else:
            return ""

    def get_row_attrs(self, instance):
        attrs = super().get_row_attrs(instance)
        if self.use_row_ordering_attributes:
            attrs["id"] = "page_%d" % instance.id
            attrs["data-w-orderable-item-id"] = instance.id
            attrs["data-w-orderable-item-label"] = instance.get_admin_display_title()
            attrs["data-w-orderable-target"] = "item"
        return attrs

    def get_context_data(self, parent_context):
        context = super().get_context_data(parent_context)
        context["show_locale_labels"] = self.show_locale_labels
        context["perms"] = parent_context.get("perms")
        context["items_count"] = parent_context.get("items_count")
        context["page_obj"] = parent_context.get("page_obj")
        context["parent_page"] = parent_context.get("parent_page")
        context["is_searching"] = parent_context.get("is_searching")
        context["is_filtering"] = parent_context.get("is_filtering")
        context["is_searching_whole_tree"] = parent_context.get(
            "is_searching_whole_tree"
        )
        context["actions_next_url"] = (
            self.actions_next_url or parent_context.get("request").path
        )
        return context



# Generated by Django 3.2.5 on 2021-07-20 12:14

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("wagtailcore", "0063_modellogentry"),
    ]

    operations = [
        migrations.AlterField(
            model_name="modellogentry",
            name="timestamp",
            field=models.DateTimeField(db_index=True, verbose_name="timestamp (UTC)"),
        ),
        migrations.AlterField(
            model_name="pagelogentry",
            name="timestamp",
            field=models.DateTimeField(db_index=True, verbose_name="timestamp (UTC)"),
        ),
    ]



import logging

from django.core.exceptions import PermissionDenied

from wagtail.log_actions import log
from wagtail.signals import unpublished

logger = logging.getLogger("wagtail")


class UnpublishPermissionError(PermissionDenied):
    """
    Raised when the object unpublish cannot be performed due to insufficient permissions.
    """

    pass


class UnpublishAction:
    def __init__(
        self,
        object,
        set_expired=False,
        commit=True,
        user=None,
        log_action=True,
    ):
        self.object = object
        self.set_expired = set_expired
        self.commit = commit
        self.user = user
        self.log_action = log_action

    def check(self, skip_permission_checks=False):
        if (
            self.user
            and not skip_permission_checks
            and not self.object.permissions_for_user(self.user).can_unpublish()
        ):
            raise UnpublishPermissionError(
                "You do not have permission to unpublish this object"
            )

    def _commit_unpublish(self, object):
        object.save()

    def _after_unpublish(self, object):
        unpublished.send(sender=type(object), instance=object)

    def _unpublish_object(self, object, set_expired, commit, user, log_action):
        """
        Unpublish the object by setting ``live`` to ``False``. Does nothing if ``live`` is already ``False``
        :param log_action: flag for logging the action. Pass False to skip logging. Can be passed an action string.
            Defaults to 'wagtail.unpublish'
        """
        if object.live:
            object.live = False
            object.has_unpublished_changes = True
            object.live_revision = None

            if set_expired:
                object.expired = True

            if commit:
                self._commit_unpublish(object)

            if log_action:
                log(
                    instance=object,
                    action=log_action
                    if isinstance(log_action, str)
                    else "wagtail.unpublish",
                    user=user,
                )

            logger.info('Unpublished: "%s" pk=%s', str(object), str(object.pk))

            object.revisions.update(approved_go_live_at=None)

            self._after_unpublish(object)

    def execute(self, skip_permission_checks=False):
        self.check(skip_permission_checks=skip_permission_checks)

        self._unpublish_object(
            self.object,
            set_expired=self.set_expired,
            commit=self.commit,
            user=self.user,
            log_action=self.log_action,
        )



# Generated by Django 4.2.1 on 2023-06-14 15:43

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("wagtailcore", "0083_workflowcontenttype"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="page",
            options={
                "permissions": [
                    ("bulk_delete_page", "Delete pages with children"),
                    ("lock_page", "Lock/unlock pages you've locked"),
                    ("publish_page", "Publish any page"),
                    ("unlock_page", "Unlock any page"),
                ],
                "verbose_name": "page",
                "verbose_name_plural": "pages",
            },
        ),
    ]



import json

from django.contrib.auth.models import AnonymousUser, Permission
from django.template import Context, Template
from django.test import TestCase, override_settings
from django.urls import reverse
from django.utils import translation
from django.utils.translation import gettext

from wagtail import hooks
from wagtail.admin.staticfiles import versioned_static
from wagtail.admin.userbar import AccessibilityItem, Userbar
from wagtail.coreutils import get_dummy_request
from wagtail.models import PAGE_TEMPLATE_VAR, Locale, Page, Site
from wagtail.test.context_processors import get_call_count, reset_call_count
from wagtail.test.testapp.models import BusinessChild, BusinessIndex, SimplePage
from wagtail.test.utils import WagtailTestUtils
from wagtail.users.models import UserProfile
from wagtail.utils.deprecation import RemovedInWagtail80Warning


class TestUserbarTag(WagtailTestUtils, TestCase):
    def setUp(self):
        self.user = self.create_superuser(
            username="test", email="test@email.com", password="password"
        )
        self.homepage = Page.objects.get(id=2)
        self.parent_page = self.homepage.get_parent()

    def dummy_request(
        self,
        user=None,
        *,
        is_preview=False,
        in_preview_panel=False,
        revision_id=None,
        is_editing=False,
    ):
        request = get_dummy_request()
        request.user = user or AnonymousUser()
        request.is_preview = is_preview
        request.is_editing = is_editing
        request.in_preview_panel = in_preview_panel
        if revision_id:
            request.revision_id = revision_id
        return request

    def test_userbar_tag(self):
        template = Template("{% load wagtailuserbar %}{% wagtailuserbar %}")
        context = Context(
            {
                PAGE_TEMPLATE_VAR: self.homepage,
                "request": self.dummy_request(self.user),
            }
        )
        with self.assertNumQueries(4):
            content = template.render(context)

        self.assertIn("<!-- Wagtail user bar embed code -->", content)
        soup = self.get_soup(content)

        css_links = soup.select("link[rel='stylesheet']")
        self.assertEqual(
            [link.get("href") for link in css_links],
            [
                # Wagtail admin core CSS should be linked with absolute URL to
                # ensure it works when loaded from a different domain
                # (e.g. headless frontend)
                f"http://localhost{versioned_static('wagtailadmin/css/core.css')}",
                # Custom CSS must be changed appropriately if necessary
                "/path/to/my/custom.css",
            ],
        )

        scripts = soup.select("script[src]")
        self.assertEqual(
            [script.get("src") for script in scripts],
            [
                # Wagtail vendor and userbar JS should be linked with absolute
                # URL to ensure it works when loaded from a different domain
                # (e.g. headless frontend)
                f"http://localhost{versioned_static('wagtailadmin/js/vendor.js')}",
                f"http://localhost{versioned_static('wagtailadmin/js/userbar.js')}",
            ],
        )

    def test_userbar_does_not_break_without_request(self):
        template = Template("{% load wagtailuserbar %}{% wagtailuserbar %}boom")
        content = template.render(Context({}))

        self.assertEqual("boom", content)

    def test_userbar_tag_self(self):
        """
        Ensure the userbar renders with `self` instead of `PAGE_TEMPLATE_VAR`
        """
        template = Template("{% load wagtailuserbar %}{% wagtailuserbar %}")
        content = template.render(
            Context(
                {
                    "self": self.homepage,
                    "request": self.dummy_request(self.user),
                }
            )
        )

        self.assertIn("<!-- Wagtail user bar embed code -->", content)

    def test_userbar_tag_anonymous_user(self):
        template = Template("{% load wagtailuserbar %}{% wagtailuserbar %}")
        content = template.render(
            Context(
                {
                    PAGE_TEMPLATE_VAR: self.homepage,
                    "request": self.dummy_request(),
                }
            )
        )

        # Make sure nothing was rendered
        self.assertEqual(content, "")

    def test_userbar_tag_no_page(self):
        template = Template("{% load wagtailuserbar %}{% wagtailuserbar %}")
        content = template.render(
            Context(
                {
                    "request": self.dummy_request(self.user),
                }
            )
        )

        self.assertIn("<!-- Wagtail user bar embed code -->", content)

        # Should render the "Go to Wagtail admin" link using an absolute URL
        soup = self.get_soup(content)
        admin_url = reverse("wagtailadmin_home")
        admin_link = soup.select_one(f"a[href='http://localhost{admin_url}']")
        self.assertIsNotNone(admin_link)
        self.assertEqual(admin_link.text.strip(), "Go to Wagtail admin")

    def test_edit_and_explore_links(self):
        template = Template("{% load wagtailuserbar %}{% wagtailuserbar %}")
        content = template.render(
            Context(
                {
                    PAGE_TEMPLATE_VAR: self.homepage,
                    "request": self.dummy_request(self.user, is_preview=False),
                }
            )
        )
        self.assertIn("<!-- Wagtail user bar embed code -->", content)
        soup = self.get_soup(content)

        edit_url = reverse("wagtailadmin_pages:edit", args=(self.homepage.id,))
        edit_link = soup.select_one(f"a[href='http://localhost{edit_url}']")
        self.assertIsNotNone(edit_link)
        self.assertEqual(edit_link.text.strip(), "Edit this page")

        explore_url = reverse("wagtailadmin_explore", args=(self.parent_page.id,))
        explore_link = soup.select_one(f"a[href='http://localhost{explore_url}']")
        self.assertIsNotNone(explore_link)
        self.assertEqual(explore_link.text.strip(), "Show in Explorer")

    def test_userbar_edit_and_explore_menu_in_previews(self):
        # The edit link should be visible on draft, revision, and workflow previews.
        # https://github.com/wagtail/wagtail/issues/10002
        template = Template("{% load wagtailuserbar %}{% wagtailuserbar %}")
        content = template.render(
            Context(
                {
                    PAGE_TEMPLATE_VAR: self.homepage,
                    "request": self.dummy_request(self.user, is_preview=True),
                }
            )
        )
        self.assertIn("<!-- Wagtail user bar embed code -->", content)
        soup = self.get_soup(content)

        edit_url = reverse("wagtailadmin_pages:edit", args=(self.homepage.id,))
        edit_link = soup.select_one(f"a[href='http://localhost{edit_url}']")
        self.assertIsNotNone(edit_link)
        self.assertEqual(edit_link.text.strip(), "Edit this page")

        explore_url = reverse("wagtailadmin_explore", args=(self.parent_page.id,))
        explore_link = soup.select_one(f"a[href='http://localhost{explore_url}']")
        self.assertIsNotNone(explore_link)
        self.assertEqual(explore_link.text.strip(), "Show in Explorer")

    def test_userbar_edit_menu_not_in_preview(self):
        # The edit link should not be visible on PreviewOnEdit/Create views.
        # https://github.com/wagtail/wagtail/issues/8765
        template = Template("{% load wagtailuserbar %}{% wagtailuserbar %}")
        content = template.render(
            Context(
                {
                    PAGE_TEMPLATE_VAR: self.homepage,
                    "request": self.dummy_request(
                        self.user, is_preview=True, is_editing=True
                    ),
                }
            )
        )
        self.assertIn("<!-- Wagtail user bar embed code -->", content)
        self.assertNotIn("Edit this page", content)
        self.assertNotIn(
            reverse("wagtailadmin_pages:edit", args=(self.homepage.id,)), content
        )

        # The explore link should still be visible
        soup = self.get_soup(content)
        explore_url = reverse("wagtailadmin_explore", args=(self.parent_page.id,))
        explore_link = soup.select_one(f"a[href='http://localhost{explore_url}']")
        self.assertIsNotNone(explore_link)
        self.assertEqual(explore_link.text.strip(), "Show in Explorer")

    def test_userbar_hidden_in_preview_panel(self):
        template = Template("{% load wagtailuserbar %}{% wagtailuserbar %}")
        content = template.render(
            Context(
                {
                    PAGE_TEMPLATE_VAR: self.homepage,
                    "request": self.dummy_request(
                        self.user, is_preview=True, in_preview_panel=True
                    ),
                }
            )
        )

        self.assertIn("<aside hidden>", content)


class TestAccessibilityCheckerConfig(WagtailTestUtils, TestCase):
    def setUp(self):
        self.user = self.login()
        self.request = get_dummy_request()
        self.request.user = self.user

    def get_content(self, position="bottom-right"):
        template = Template(
            f"{{% load wagtailuserbar %}}{{% wagtailuserbar position='{position}' %}}"
        )
        content = template.render(Context({"request": self.request}))
        return self.get_soup(content)

    def get_script(self):
        soup = self.get_content()
        # Should include the configuration as a JSON script with the specific id
        return soup.find("script", id="accessibility-axe-configuration")

    def get_config(self):
        return json.loads(self.get_script().string)

    def get_hook(self, item_class):
        def customise_accessibility_checker(request, items, page):
            items[:] = [
                item_class() if isinstance(item, AccessibilityItem) else item
                for item in items
            ]

        return customise_accessibility_checker

    def test_position(self):
        content = self.get_content(position="top-left")
        # The element with the data-wagtail-userbar attribute should have the
        # right CSS class (w-userbar--top-left) for the positioning
        target = content.select_one("aside [data-wagtail-userbar]")
        self.assertIsNotNone(target)
        self.assertIn("w-userbar--top-left", target.get("class"))

        # The accessibility results dialog should be teleported to the
        # [data-wagtail-userbar] element so that it is positioned correctly
        dialog_content = content.select_one("#accessibility-results")
        self.assertIsNotNone(dialog_content)
        template = dialog_content.parent
        self.assertIsNotNone(template)
        self.assertEqual(
            template.get("data-w-teleport-target-value"),
            "[data-wagtail-userbar]",
        )

    def test_config_json(self):
        script = self.get_script()
        # The configuration should be a valid non-empty JSON script
        self.assertIsNotNone(script)
        self.assertEqual(script.attrs["type"], "application/json")
        config_string = script.string.strip()
        self.assertGreater(len(config_string), 0)
        config = json.loads(config_string)
        self.assertIsInstance(config, dict)
        self.assertGreater(len(config.keys()), 0)

    def test_messages(self):
        # Should include Wagtail's error messages
        config = self.get_config()
        self.assertIsInstance(config.get("messages"), dict)
        self.assertEqual(
            config["messages"]["empty-heading"]["error_name"],
            "Empty heading found",
        )
        self.assertEqual(
            config["messages"]["empty-heading"]["help_text"],
            "Use meaningful text for screen reader users",
        )

    def test_custom_message(self):
        class CustomMessageAccessibilityItem(AccessibilityItem):
            # Override via class attribute
            axe_messages = {
                "empty-heading": {
                    "error_name": "Headings should not be empty!",
                    "help_text": "Use meaningful text!",
                },
            }

            # Override via method
            def get_axe_messages(self, request):
                return {
                    **super().get_axe_messages(request),
                    "color-contrast-enhanced": {
                        "error_name": "Insufficient colour contrast!",
                        "help_text": "Ensure contrast ratio of at least 4.5:1",
                    },
                }

        with hooks.register_temporarily(
            "construct_wagtail_userbar",
            self.get_hook(CustomMessageAccessibilityItem),
        ):
            config = self.get_config()
            self.assertEqual(
                config["messages"],
                {
                    "empty-heading": {
                        "error_name": "Headings should not be empty!",
                        "help_text": "Use meaningful text!",
                    },
                    "color-contrast-enhanced": {
                        "error_name": "Insufficient colour contrast!",
                        "help_text": "Ensure contrast ratio of at least 4.5:1",
                    },
                },
            )

    def test_unset_run_only(self):
        class UnsetRunOnlyAccessibilityItem(AccessibilityItem):
            # Example config that unsets the runOnly property so that all
            # non-experimental rules are run, but the experimental
            # focus-order-semantics rule is explicitly enabled
            axe_run_only = None
            axe_rules = {"focus-order-semantics": {"enabled": True}}

        with hooks.register_temporarily(
            "construct_wagtail_userbar",
            self.get_hook(UnsetRunOnlyAccessibilityItem),
        ):
            config = self.get_config()
            self.assertEqual(
                config["options"],
                # Should not include the runOnly property, but should include
                # the focus-order-semantics rule
                {"rules": {"focus-order-semantics": {"enabled": True}}},
            )

    def test_custom_context(self):
        class CustomContextAccessibilityItem(AccessibilityItem):
            axe_include = ["article", "section"]
            axe_exclude = [".sr-only"]

            def get_axe_exclude(self, request):
                return [*super().get_axe_exclude(request), "[data-please-ignore]"]

        with hooks.register_temporarily(
            "construct_wagtail_userbar",
            self.get_hook(CustomContextAccessibilityItem),
        ):
            config = self.get_config()
            self.assertEqual(
                config["context"],
                {
                    # Override via class attribute
                    "include": ["article", "section"],
                    "exclude": [
                        # Override via class attribute
                        ".sr-only",
                        # Should include the default exclude selectors
                        {"fromShadowDom": ["wagtail-userbar"]},
                        # Override via method
                        "[data-please-ignore]",
                    ],
                },
            )

    def test_custom_run_only_and_rules_per_request(self):
        class CustomRunOnlyAccessibilityItem(AccessibilityItem):
            # Enable all rules within these tags
            axe_run_only = [
                "wcag2a",
                "wcag2aa",
                "wcag2aaa",
                "wcag21a",
                "wcag21aa",
                "wcag22aa",
                "best-practice",
            ]
            # Turn off the color-contrast-enhanced rule
            axe_rules = {
                "color-contrast-enhanced": {"enabled": False},
            }

            def get_axe_rules(self, request):
                # Do not turn off any rules for superusers
                if request.user.is_superuser:
                    return {}
                return super().get_axe_rules(request)

        with hooks.register_temporarily(
            "construct_wagtail_userbar",
            self.get_hook(CustomRunOnlyAccessibilityItem),
        ):
            config = self.get_config()
            self.assertEqual(
                config["options"],
                {
                    "runOnly": CustomRunOnlyAccessibilityItem.axe_run_only,
                    "rules": {},
                },
            )

            self.user.is_superuser = False
            self.user.user_permissions.add(
                Permission.objects.get(
                    content_type__app_label="wagtailadmin", codename="access_admin"
                )
            )
            self.user.save()

            config = self.get_config()
            self.assertEqual(
                config["options"],
                {
                    "runOnly": CustomRunOnlyAccessibilityItem.axe_run_only,
                    "rules": CustomRunOnlyAccessibilityItem.axe_rules,
                },
            )

    def test_custom_rules_and_checks(self):
        class CustomRulesAndChecksAccessibilityItem(AccessibilityItem):
            # Override via class attribute
            axe_custom_checks = [
                {
                    "id": "check-image-alt-text",
                    "options": {"pattern": "\\.[a-z]{1,4}$|_"},
                },
            ]

            # Add via method
            def get_axe_custom_rules(self, request):
                return super().get_axe_custom_rules(request) + [
                    {
                        "id": "link-text-quality",
                        "impact": "serious",
                        "selector": "a",
                        "tags": ["best-practice"],
                        "any": ["check-link-text"],
                        "enabled": True,
                    }
                ]

            def get_axe_custom_checks(self, request):
                return super().get_axe_custom_checks(request) + [
                    {
                        "id": "check-link-text",
                        "options": {"pattern": "learn more$"},
                    }
                ]

        with hooks.register_temporarily(
            "construct_wagtail_userbar",
            self.get_hook(CustomRulesAndChecksAccessibilityItem),
        ):
            self.maxDiff = None
            config = self.get_config()
            self.assertEqual(
                config["spec"],
                {
                    "rules": [
                        {
                            "id": "alt-text-quality",
                            "impact": "serious",
                            "selector": "img[alt]",
                            "tags": ["best-practice"],
                            "any": ["check-image-alt-text"],
                            "enabled": True,
                        },
                        {
                            "id": "link-text-quality",
                            "impact": "serious",
                            "selector": "a",
                            "tags": ["best-practice"],
                            "any": ["check-link-text"],
                            "enabled": True,
                        },
                    ],
                    "checks": [
                        {
                            "id": "check-image-alt-text",
                            "options": {"pattern": "\\.[a-z]{1,4}$|_"},
                        },
                        {
                            "id": "check-link-text",
                            "options": {"pattern": "learn more$"},
                        },
                    ],
                },
            )


class TestUserbarInPageServe(WagtailTestUtils, TestCase):
    def setUp(self):
        self.user = self.login()
        self.request = get_dummy_request(site=Site.objects.first())
        self.request.user = self.user
        self.homepage = Page.objects.get(id=2).specific
        # Use a specific page model to use our template that has {% wagtailuserbar %}
        self.page = SimplePage(title="Rendang", content="Enak", live=True)
        self.homepage.add_child(instance=self.page)

    def test_userbar_rendered(self):
        reset_call_count()
        response = self.page.serve(self.request)
        response.render()

        self.assertEqual(response.status_code, 200)
        self.assertContains(response, '<template id="wagtail-userbar-template">')
        self.assertEqual(get_call_count(), 1)

    def test_userbar_anonymous_user_cannot_see(self):
        self.request.user = AnonymousUser()
        response = self.page.serve(self.request)
        response.render()

        # Check that the userbar is not rendered
        self.assertEqual(response.status_code, 200)
        self.assertNotContains(response, '<template id="wagtail-userbar-template">')

    def test_construct_wagtail_userbar_hook_passes_page(self):
        kwargs = {}

        def construct_wagtail_userbar(request, items, page):
            kwargs["page"] = page
            return items

        with hooks.register_temporarily(
            "construct_wagtail_userbar",
            construct_wagtail_userbar,
        ):
            response = self.page.serve(self.request)
            response.render()

            self.assertEqual(kwargs.get("page"), self.page)

    def test_deprecated_construct_wagtail_userbar_hook_without_page(self):
        kwargs = {}

        def construct_wagtail_userbar(request, items):
            kwargs["called"] = True
            return items

        with (
            self.assertWarnsMessage(
                RemovedInWagtail80Warning,
                "`construct_wagtail_userbar` hook functions should accept a "
                "`page` argument in third position",
            ),
            hooks.register_temporarily(
                "construct_wagtail_userbar",
                construct_wagtail_userbar,
            ),
        ):
            response = self.page.serve(self.request)
            response.render()

            self.assertTrue(kwargs.get("called"))

    @override_settings(
        WAGTAIL_I18N_ENABLED=True,
        WAGTAIL_CONTENT_LANGUAGES=[("en", "English"), ("fr", "French")],
        WAGTAILADMIN_PERMITTED_LANGUAGES=[("en", "English")],
        LANGUAGE_CODE="en",
    )
    def test_userbar_rendered_in_admin_permitted_language_if_user_has_no_language(self):
        french = Locale.objects.create(language_code="fr")
        self.homepage.copy_for_translation(french)
        french_page = self.page.copy_for_translation(french)

        with translation.override("fr"):
            response = french_page.serve(self.request)
            response.render()

        self.assertContains(response, "Go to Wagtail admin")

    @override_settings(
        WAGTAIL_I18N_ENABLED=True,
        WAGTAIL_CONTENT_LANGUAGES=[("en", "English"), ("fr", "French")],
        LANGUAGE_CODE="en",
    )
    def test_userbar_rendered_in_active_language_if_admin_permitted(self):
        french = Locale.objects.create(language_code="fr")
        self.homepage.copy_for_translation(french)
        french_page = self.page.copy_for_translation(french)

        with translation.override("fr"):
            response = french_page.serve(self.request)
            response.render()

        with translation.override("fr"):
            expected_text = gettext("Go to Wagtail admin")
        self.assertContains(response, expected_text)

    @override_settings(LANGUAGE_CODE="en")
    def test_userbar_rendered_in_user_preferred_language(self):
        profile = UserProfile.get_for_user(self.user)
        profile.preferred_language = "fr"
        profile.save()

        response = self.page.serve(self.request)
        response.render()

        # Ensure we have a French string without tests failing if translation changes
        with translation.override("fr"):
            expected_text = gettext("Go to Wagtail admin")
        self.assertNotEqual(expected_text, "Go to Wagtail admin")
        self.assertContains(response, expected_text)


class TestUserbarHooksForChecksPanel(WagtailTestUtils, TestCase):
    def setUp(self):
        self.user = self.login()
        self.homepage = Page.objects.get(id=2).specific

    def test_construct_wagtail_userbar_hook_passes_page(self):
        kwargs = {}

        def construct_wagtail_userbar(request, items, page):
            kwargs["called"] = True
            return items

        with hooks.register_temporarily(
            "construct_wagtail_userbar",
            construct_wagtail_userbar,
        ):
            response = self.client.get(
                reverse("wagtailadmin_pages:edit", args=(self.homepage.id,))
            )

        self.assertEqual(response.status_code, 200)
        self.assertTrue(kwargs.get("called"))

    def test_deprecated_construct_wagtail_userbar_hook_without_page(self):
        kwargs = {}

        def construct_wagtail_userbar(request, items):
            kwargs["called"] = True
            return items

        with (
            self.assertWarnsMessage(
                RemovedInWagtail80Warning,
                "`construct_wagtail_userbar` hook functions should accept a "
                "`page` argument in third position",
            ),
            hooks.register_temporarily(
                "construct_wagtail_userbar",
                construct_wagtail_userbar,
            ),
        ):
            response = self.client.get(
                reverse("wagtailadmin_pages:edit", args=(self.homepage.id,))
            )

        self.assertEqual(response.status_code, 200)
        self.assertTrue(kwargs.get("called"))


class TestUserbarAddLink(WagtailTestUtils, TestCase):
    fixtures = ["test.json"]

    def setUp(self):
        self.user = self.login()
        self.request = get_dummy_request(site=Site.objects.first())
        self.request.user = self.user
        self.homepage = Page.objects.get(url_path="/home/")
        self.event_index = Page.objects.get(url_path="/home/events/").specific

        self.business_index = BusinessIndex(title="Business", live=True)
        self.homepage.add_child(instance=self.business_index)

        self.business_child = BusinessChild(title="Business Child", live=True)
        self.business_index.add_child(instance=self.business_child)

    def test_page_allowing_subpages(self):
        response = self.event_index.serve(self.request)
        response.render()
        self.assertEqual(response.status_code, 200)

        # page allows subpages, so the 'add page' button should show
        expected_url = self.request.build_absolute_uri(
            reverse("wagtailadmin_pages:add_subpage", args=(self.event_index.id,))
        )
        needle = f"""
            <a href="{expected_url}" target="_parent" role="menuitem">
                <svg class="icon icon-plus w-action-icon" aria-hidden="true">
                    <use href="#icon-plus"></use>
                </svg>
                Add a child page
            </a>
            """
        self.assertTagInHTML(needle, response.content.decode())

    def test_page_disallowing_subpages(self):
        response = self.business_child.serve(self.request)
        response.render()
        self.assertEqual(response.status_code, 200)

        # page disallows subpages, so the 'add page' button shouldn't show
        expected_url = reverse(
            "wagtailadmin_pages:add_subpage", args=(self.business_index.id,)
        )
        soup = self.get_soup(response.content)
        link = soup.find("a", attrs={"href": expected_url})
        self.assertIsNone(link)


class TestUserbarComponent(WagtailTestUtils, TestCase):
    def setUp(self):
        self.request = get_dummy_request()
        self.request.user = self.create_superuser(
            username="test", email="test@email.com", password="password"
        )
        self.homepage = Page.objects.get(id=2)
        self.parent_page = self.homepage.get_parent()

    def test_render_anonymous_user(self):
        self.request.user = AnonymousUser()
        rendered = Userbar().render_html({"request": self.request})
        soup = self.get_soup(rendered)

        items = soup.select("li")
        self.assertEqual(len(items), 2)

        admin_url = f"http://localhost{reverse('wagtailadmin_home')}"
        admin_item = items[0]
        admin_link = admin_item.select_one("a")
        self.assertIsNotNone(admin_link)
        self.assertEqual(admin_link.get("href"), admin_url)
        self.assertEqual(admin_link.text.strip(), "Go to Wagtail admin")

        accessibility_item = items[-1]
        button = accessibility_item.select_one("button")
        self.assertIsNotNone(button)
        self.assertEqual(
            button.get_text(separator=" | ", strip=True).strip(),
            "Issues found | Accessibility",
        )

    def test_render_no_request(self):
        rendered = Userbar().render_html({})
        soup = self.get_soup(rendered)
        # Without a request provided, URLs should fall back to the
        # WAGTAILADMIN_BASE_URL setting
        base_url = "http://testserver"

        userbar = soup.select_one("[data-wagtail-userbar]")
        self.assertIsNotNone(userbar)
        self.assertEqual(userbar.get("data-wagtail-userbar-origin"), base_url)

        items = soup.select("li")
        self.assertEqual(len(items), 2)

        admin_url = f"{base_url}{reverse('wagtailadmin_home')}"
        admin_item = items[0]
        admin_link = admin_item.select_one("a")
        self.assertIsNotNone(admin_link)
        self.assertEqual(admin_link.get("href"), admin_url)
        self.assertEqual(admin_link.text.strip(), "Go to Wagtail admin")

        accessibility_item = items[-1]
        button = accessibility_item.select_one("button")
        self.assertIsNotNone(button)
        self.assertEqual(
            button.get_text(separator=" | ", strip=True).strip(),
            "Issues found | Accessibility",
        )

        css_links = soup.select("link[rel='stylesheet']")
        self.assertEqual(
            [link.get("href") for link in css_links],
            [
                # Wagtail admin core CSS should be linked with absolute URL to
                # ensure it works when loaded from a different domain
                # (e.g. headless frontend)
                f"{base_url}{versioned_static('wagtailadmin/css/core.css')}",
                # Custom CSS must be changed appropriately if necessary
                "/path/to/my/custom.css",
            ],
        )

        scripts = soup.select("script[src]")
        self.assertEqual(
            [script.get("src") for script in scripts],
            [
                # Wagtail vendor and userbar JS should be linked with absolute
                # URL to ensure it works when loaded from a different domain
                # (e.g. headless frontend)
                f"{base_url}{versioned_static('wagtailadmin/js/vendor.js')}",
                f"{base_url}{versioned_static('wagtailadmin/js/userbar.js')}",
            ],
        )

    def test_render_minimal(self):
        rendered = Userbar().render_html({"request": self.request})
        soup = self.get_soup(rendered)

        userbar = soup.select_one("[data-wagtail-userbar]")
        self.assertIsNotNone(userbar)
        # The origin should be based on the request's scheme and host information
        self.assertEqual(userbar.get("data-wagtail-userbar-origin"), "http://localhost")

        items = soup.select("li")
        self.assertEqual(len(items), 2)

        admin_url = f"http://localhost{reverse('wagtailadmin_home')}"
        admin_item = items[0]
        admin_link = admin_item.select_one("a")
        self.assertIsNotNone(admin_link)
        self.assertEqual(admin_link.get("href"), admin_url)
        self.assertEqual(admin_link.text.strip(), "Go to Wagtail admin")

        accessibility_item = items[-1]
        button = accessibility_item.select_one("button")
        self.assertIsNotNone(button)
        self.assertEqual(
            button.get_text(separator=" | ", strip=True).strip(),
            "Issues found | Accessibility",
        )

    def test_render_with_page(self):
        rendered = Userbar(object=self.homepage).render_html(
            {"request": self.request, PAGE_TEMPLATE_VAR: self.homepage}
        )
        soup = self.get_soup(rendered)

        links = soup.select("li a")
        expected_urls = [
            reverse("wagtailadmin_home"),
            reverse("wagtailadmin_explore", args=(self.parent_page.id,)),
            reverse("wagtailadmin_pages:edit", args=(self.homepage.id,)),
            reverse("wagtailadmin_pages:add_subpage", args=(self.homepage.id,)),
        ]
        self.assertEqual(len(links), 4)
        self.assertEqual(
            [link.get("href") for link in links],
            [f"http://localhost{url}" for url in expected_urls],
        )

        accessibility_button = soup.select_one("li button")
        self.assertIsNotNone(accessibility_button)
        self.assertEqual(
            accessibility_button.get_text(separator=" | ", strip=True).strip(),
            "Issues found | Accessibility",
        )

    @override_settings(WAGTAILADMIN_BASE_URL=None)
    def test_render_without_request_and_admin_base_url_setting(self):
        rendered = Userbar().render_html({})
        soup = self.get_soup(rendered)

        userbar = soup.select_one("[data-wagtail-userbar]")
        self.assertIsNotNone(userbar)
        # Default to an empty string instead of rendering None,
        # so the JS code can provide the fallback
        self.assertEqual(userbar.get("data-wagtail-userbar-origin"), "")

        items = soup.select("li")
        self.assertEqual(len(items), 2)

        # Becomes the root-relative URL since no request is provided
        # and WAGTAILADMIN_BASE_URL is None
        admin_url = reverse("wagtailadmin_home")
        admin_item = items[0]
        admin_link = admin_item.select_one("a")
        self.assertIsNotNone(admin_link)
        self.assertEqual(admin_link.get("href"), admin_url)
        self.assertEqual(admin_link.text.strip(), "Go to Wagtail admin")

        accessibility_item = items[-1]
        button = accessibility_item.select_one("button")
        self.assertIsNotNone(button)
        self.assertEqual(
            button.get_text(separator=" | ", strip=True).strip(),
            "Issues found | Accessibility",
        )

        css_links = soup.select("link[rel='stylesheet']")
        self.assertEqual(
            [link.get("href") for link in css_links],
            [
                # Cannot build absolute URL without a request and
                # WAGTAILADMIN_BASE_URL, so should be root-relative
                versioned_static("wagtailadmin/css/core.css"),
                "/path/to/my/custom.css",
            ],
        )

        scripts = soup.select("script[src]")
        self.assertEqual(
            [script.get("src") for script in scripts],
            [
                # Cannot build absolute URL without a request and
                # WAGTAILADMIN_BASE_URL, so should be root-relative
                versioned_static("wagtailadmin/js/vendor.js"),
                versioned_static("wagtailadmin/js/userbar.js"),
            ],
        )



import json
import re
from html import unescape

from bs4 import BeautifulSoup
from django import forms
from django.test import TestCase
from django.test.utils import override_settings
from django.utils.html import escape

from wagtail.admin import widgets
from wagtail.admin.forms.tags import TagField
from wagtail.models import Locale, Page
from wagtail.test.testapp.forms import AdminStarDateInput
from wagtail.test.testapp.models import EventPage, RestaurantTag, SimplePage
from wagtail.utils.deprecation import RemovedInWagtail80Warning


class TestAdminPageChooserWidget(TestCase):
    @classmethod
    def setUpTestData(cls):
        cls.root_page = Page.objects.get(id=2)

        # Add child page
        cls.child_page = SimplePage(
            title="foobarbaz",
            content="hello",
        )
        cls.root_page.add_child(instance=cls.child_page)

    def test_not_hidden(self):
        widget = widgets.AdminPageChooser()
        self.assertFalse(widget.is_hidden)

    def test_adapt(self):
        widget = widgets.AdminPageChooser()

        js_args = widgets.PageChooserAdapter().js_args(widget)
        self.assertInHTML(
            """<input id="__ID__" name="__NAME__" type="hidden" />""", js_args[0]
        )
        self.assertIn("Choose a page", js_args[0])
        self.assertEqual(js_args[1], "__ID__")
        self.assertEqual(
            js_args[2],
            {
                "canChooseRoot": False,
                "modelNames": ["wagtailcore.page"],
                "userPerms": None,
                "modalUrl": "/admin/choose-page/",
            },
        )

    def test_adapt_with_target_model(self):
        widget = widgets.AdminPageChooser(target_models=[SimplePage, EventPage])

        js_args = widgets.PageChooserAdapter().js_args(widget)
        self.assertEqual(
            js_args[2]["modelNames"], ["tests.simplepage", "tests.eventpage"]
        )

    def test_adapt_with_can_choose_root(self):
        widget = widgets.AdminPageChooser(can_choose_root=True)

        js_args = widgets.PageChooserAdapter().js_args(widget)
        self.assertTrue(js_args[2]["canChooseRoot"])

    def test_render_html(self):
        # render_html is mostly an internal API, but we do want to support calling it with None as
        # a value, to render a blank field without the JS initialiser (so that we can call that
        # separately in our own context and hold on to the return value)
        widget = widgets.AdminPageChooser()

        html = widget.render_html("test", None, {})
        self.assertInHTML("""<input name="test" type="hidden" />""", html)
        self.assertIn("Choose a page", html)

    def test_render_js_init(self):
        widget = widgets.AdminPageChooser()

        html = widget.render("test", None, {"id": "test-id"})
        self.assertIn(
            'new PageChooser("test-id", {"modelNames": ["wagtailcore.page"], "canChooseRoot": false, "userPerms": null, "modalUrl": "/admin/choose-page/"});',
            html,
        )

    def test_render_js_init_with_user_perm(self):
        widget = widgets.AdminPageChooser(user_perms="copy_to")

        html = widget.render("test", None, {"id": "test-id"})
        self.assertIn(
            'new PageChooser("test-id", {"modelNames": ["wagtailcore.page"], "canChooseRoot": false, "userPerms": "copy_to", "modalUrl": "/admin/choose-page/"});',
            html,
        )

    def test_render_with_value(self):
        widget = widgets.AdminPageChooser()

        html = widget.render("test", self.child_page, {"id": "test-id"})
        self.assertInHTML(
            """<input id="test-id" name="test" type="hidden" value="%d" />"""
            % self.child_page.id,
            html,
        )
        # SimplePage has a custom get_admin_display_title method which should be reflected here
        self.assertInHTML("foobarbaz (simple page)", html)

        self.assertIn(
            'new PageChooser("test-id", {"modelNames": ["wagtailcore.page"], "canChooseRoot": false, "userPerms": null, "modalUrl": "/admin/choose-page/", "parentId": %d});'
            % self.root_page.id,
            html,
        )

    def test_render_with_target_model(self):
        widget = widgets.AdminPageChooser(target_models=[SimplePage])

        html = widget.render("test", None, {"id": "test-id"})
        self.assertIn(
            'new PageChooser("test-id", {"modelNames": ["tests.simplepage"], "canChooseRoot": false, "userPerms": null, "modalUrl": "/admin/choose-page/"});',
            html,
        )

        html = widget.render("test", self.child_page, {"id": "test-id"})
        self.assertIn("Choose a page (Simple page)", html)

    def test_render_with_target_model_as_single_instance(self):
        widget = widgets.AdminPageChooser(target_models=SimplePage)

        html = widget.render("test", None, {"id": "test-id"})
        self.assertIn(
            'new PageChooser("test-id", {"modelNames": ["tests.simplepage"], "canChooseRoot": false, "userPerms": null, "modalUrl": "/admin/choose-page/"});',
            html,
        )

        html = widget.render("test", self.child_page, {"id": "test-id"})
        self.assertIn("Choose a page (Simple page)", html)

    def test_render_with_target_model_as_single_string(self):
        widget = widgets.AdminPageChooser(target_models="tests.SimplePage")

        html = widget.render("test", None, {"id": "test-id"})
        self.assertIn(
            'new PageChooser("test-id", {"modelNames": ["tests.simplepage"], "canChooseRoot": false, "userPerms": null, "modalUrl": "/admin/choose-page/"});',
            html,
        )

        html = widget.render("test", self.child_page, {"id": "test-id"})
        self.assertIn("Choose a page (Simple page)", html)

    def test_render_with_multiple_target_models(self):
        target_models = [SimplePage, "tests.eventpage"]
        widget = widgets.AdminPageChooser(target_models=target_models)

        html = widget.render("test", None, {"id": "test-id"})
        self.assertIn(
            'new PageChooser("test-id", {"modelNames": ["tests.simplepage", "tests.eventpage"], "canChooseRoot": false, "userPerms": null, "modalUrl": "/admin/choose-page/"});',
            html,
        )

        html = widget.render("test", self.child_page, {"id": "test-id"})
        self.assertIn("Choose a page", html)

    def test_render_js_init_with_can_choose_root(self):
        widget = widgets.AdminPageChooser(can_choose_root=True)

        html = widget.render("test", self.child_page, {"id": "test-id"})
        self.assertIn(
            'new PageChooser("test-id", {"modelNames": ["wagtailcore.page"], "canChooseRoot": true, "userPerms": null, "modalUrl": "/admin/choose-page/", "parentId": %d});'
            % self.root_page.id,
            html,
        )

    def test_get_instance(self):
        widget = widgets.AdminPageChooser(target_models=[SimplePage])
        self.assertIsNone(widget.get_instance(None))
        self.assertIsNone(widget.get_instance(self.root_page.id))
        self.assertIsNone(widget.get_instance(self.child_page.id + 100))
        self.assertEqual(widget.get_instance(self.child_page), self.child_page)
        self.assertEqual(widget.get_instance(self.child_page.id), self.child_page)


class TestAdminDateInput(TestCase):
    def test_adapt(self):
        widget = widgets.AdminDateInput()

        js_args = widgets.AdminDateInputAdapter().js_args(widget)

        self.assertEqual(js_args[0], {"dayOfWeekStart": 0, "format": "Y-m-d"})

    def test_adapt_with_custom_format(self):
        widget = widgets.AdminDateInput(format="%d.%m.%Y")

        js_args = widgets.AdminDateInputAdapter().js_args(widget)

        self.assertEqual(js_args[0], {"dayOfWeekStart": 0, "format": "d.m.Y"})

    def test_render_js_init(self):
        widget = widgets.AdminDateInput()

        html = widget.render("test", None, attrs={"id": "test-id"})

        self.assertInHTML(
            '<input type="text" name="test" autocomplete="off" id="test-id" />', html
        )

        # we should see the JS initialiser code:
        # initDateChooser("test-id", {"dayOfWeekStart": 0, "format": "Y-m-d"});
        # except that we can't predict the order of the config options
        self.assertIn('initDateChooser("test\\u002Did", {', html)
        self.assertIn('"dayOfWeekStart": 0', html)
        self.assertIn('"format": "Y-m-d"', html)

    def test_render_js_init_with_format(self):
        widget = widgets.AdminDateInput(format="%d.%m.%Y.")

        html = widget.render("test", None, attrs={"id": "test-id"})
        self.assertIn(
            '"format": "d.m.Y."',
            html,
        )

    @override_settings(WAGTAIL_DATE_FORMAT="%d.%m.%Y.")
    def test_render_js_init_with_format_from_settings(self):
        widget = widgets.AdminDateInput()

        html = widget.render("test", None, attrs={"id": "test-id"})
        self.assertIn(
            '"format": "d.m.Y."',
            html,
        )

    def test_media_inheritance(self):
        """
        Widgets inheriting from AdminDateInput should have their media definitions merged
        with AdminDateInput's
        """
        widget = AdminStarDateInput()
        media_html = str(widget.media)
        self.assertIn("wagtailadmin/js/date-time-chooser.js", media_html)
        self.assertIn("vendor/star_date.js", media_html)


class TestAdminTimeInput(TestCase):
    def test_adapt(self):
        widget = widgets.AdminTimeInput()

        js_args = widgets.AdminTimeInputAdapter().js_args(widget)

        self.assertEqual(js_args[0], {"format": "H:i", "formatTime": "H:i"})

    def test_adapt_with_custom_format(self):
        widget = widgets.AdminTimeInput(format="%H:%M:%S")

        js_args = widgets.AdminTimeInputAdapter().js_args(widget)

        self.assertEqual(js_args[0], {"format": "H:i:s", "formatTime": "H:i:s"})

    def test_render_js_init(self):
        widget = widgets.AdminTimeInput()

        html = widget.render("test", None, attrs={"id": "test-id"})

        self.assertInHTML(
            '<input type="text" name="test" autocomplete="off" id="test-id" />', html
        )

        # we should see the JS initialiser code:
        # initDateChooser("test-id", {"dayOfWeekStart": 0, "format": "Y-m-d"});
        # except that we can't predict the order of the config options
        self.assertIn('initTimeChooser("test\\u002Did", {', html)
        self.assertIn('"format": "H:i"', html)

    def test_render_js_init_with_format(self):
        widget = widgets.AdminTimeInput(format="%H:%M:%S")

        html = widget.render("test", None, attrs={"id": "test-id"})
        self.assertIn(
            '"format": "H:i:s"',
            html,
        )

    @override_settings(WAGTAIL_TIME_FORMAT="%H:%M:%S")
    def test_render_js_init_with_format_from_settings(self):
        widget = widgets.AdminTimeInput()

        html = widget.render("test", None, attrs={"id": "test-id"})
        self.assertIn(
            '"format": "H:i:s"',
            html,
        )


class TestAdminDateTimeInput(TestCase):
    def test_adapt(self):
        widget = widgets.AdminDateTimeInput()

        js_args = widgets.AdminDateTimeInputAdapter().js_args(widget)

        self.assertEqual(
            js_args[0],
            {
                "dayOfWeekStart": 0,
                "format": "Y-m-d H:i",
                "formatTime": "H:i",
                "parentID": "body",
            },
        )

    def test_adapt_with_custom_format(self):
        widget = widgets.AdminDateTimeInput(
            format="%d.%m.%Y. %H:%M", time_format="%H:%M %p"
        )

        js_args = widgets.AdminDateTimeInputAdapter().js_args(widget)

        self.assertEqual(
            js_args[0],
            {
                "dayOfWeekStart": 0,
                "format": "d.m.Y. H:i",
                "formatTime": "H:i A",
                "parentID": "body",
            },
        )

    def test_adapt_with_custom_parent_selector(self):
        widget = widgets.AdminDateTimeInput(
            js_overlay_parent_selector="#test-parent-id"
        )

        js_args = widgets.AdminDateTimeInputAdapter().js_args(widget)

        self.assertEqual(
            js_args[0],
            {
                "dayOfWeekStart": 0,
                "format": "Y-m-d H:i",
                "formatTime": "H:i",
                "parentID": "#test-parent-id",
            },
        )

    def test_render_js_init(self):
        widget = widgets.AdminDateTimeInput()

        html = widget.render("test", None, attrs={"id": "test-id"})

        self.assertInHTML(
            '<input type="text" name="test" autocomplete="off" id="test-id" />', html
        )

        # we should see the JS initialiser code:
        # initDateTimeChooser("test-id", {"dayOfWeekStart": 0, "format": "Y-m-d H:i"});
        # except that we can't predict the order of the config options
        self.assertIn('initDateTimeChooser("test\\u002Did", {', html)
        self.assertIn('"dayOfWeekStart": 0', html)
        self.assertIn('"format": "Y-m-d H:i"', html)
        self.assertIn('"formatTime": "H:i"', html)
        self.assertIn('"parentID": "body"', html)

    def test_render_js_init_with_format(self):
        widget = widgets.AdminDateTimeInput(
            format="%d.%m.%Y. %H:%M", time_format="%H:%M %p"
        )

        html = widget.render("test", None, attrs={"id": "test-id"})
        self.assertIn(
            '"format": "d.m.Y. H:i"',
            html,
        )
        self.assertIn(
            '"formatTime": "H:i A"',
            html,
        )

    @override_settings(
        WAGTAIL_DATETIME_FORMAT="%d.%m.%Y. %H:%M", WAGTAIL_TIME_FORMAT="%H:%M %p"
    )
    def test_render_js_init_with_format_from_settings(self):
        widget = widgets.AdminDateTimeInput()

        html = widget.render("test", None, attrs={"id": "test-id"})
        self.assertIn(
            '"format": "d.m.Y. H:i"',
            html,
        )
        self.assertIn(
            '"formatTime": "H:i A"',
            html,
        )


class TestAdminTagWidget(TestCase):
    def get_js_init_params(self, html):
        """
        Returns a list of the key parts of data needed for the w-tag controlled element
        An id for the element with the 'w-tag' controller, the autocomplete url & tag options

        example element <input data-controller="w-tag" id="test_id" data-w-tag-url-value="/admin/tag-autocomplete/" data-w-tag-options-value="{...encoded json opts}" />
        example result - ["test_id", "/admin/tag-autocomplete/", {'allowSpaces': True}]
        """

        element_id = re.search(
            r'data-controller=\"w-tag\" id=\"((?:\\.|[^"\\])*)\"\s+', html
        ).group(1)
        autocomplete_url = re.search(
            r'data-w-tag-url-value=\"((?:\\.|[^"\\])*)"', html
        ).group(1)
        options = re.search(
            r'data-w-tag-options-value=\"((?:\\.|[^"\\])*)"', html
        ).group(1)

        return [
            element_id,
            autocomplete_url,
            json.loads(unescape(options)),
        ]

    def test_render_js_init_basic(self):
        """Checks that the 'w-tag' controller attributes are correctly added to the tag widgets"""
        widget = widgets.AdminTagWidget()

        html = widget.render("tags", None, attrs={"id": "alpha"})
        params = self.get_js_init_params(html)

        self.assertEqual(
            params,
            [
                "alpha",
                "/admin/tag-autocomplete/",
                {"allowSpaces": True, "tagLimit": None, "autocompleteOnly": False},
            ],
        )

    @override_settings(WAGTAIL_TAG_SPACES_ALLOWED=False)
    def test_render_js_init_no_spaces_allowed(self):
        """Checks that the 'w-tag' controller attributes are correctly added to the tag widgets based  on WAGTAIL_TAG_SPACES_ALLOWED in settings"""
        widget = widgets.AdminTagWidget()

        html = widget.render("tags", None, attrs={"id": "alpha"})
        params = self.get_js_init_params(html)

        self.assertEqual(
            params,
            [
                "alpha",
                "/admin/tag-autocomplete/",
                {"allowSpaces": False, "tagLimit": None, "autocompleteOnly": False},
            ],
        )

    @override_settings(WAGTAIL_TAG_LIMIT=5)
    def test_render_js_init_with_tag_limit(self):
        """Checks that the 'w-tag' controller attributes are correctly added to the tag widget using options based on WAGTAIL_TAG_LIMIT in settings"""

        widget = widgets.AdminTagWidget()

        html = widget.render("tags", None, attrs={"id": "alpha"})
        params = self.get_js_init_params(html)

        self.assertEqual(
            params,
            [
                "alpha",
                "/admin/tag-autocomplete/",
                {"allowSpaces": True, "tagLimit": 5, "autocompleteOnly": False},
            ],
        )

    def test_render_js_init_with_tag_model(self):
        """
        Checks that the 'w-tag' controller attributes are correctly added to the tag widget using
        the correct autocomplete URL for the custom model,
        and sets autocompleteOnly according to that model's free_tagging attribute
        """
        widget = widgets.AdminTagWidget(tag_model=RestaurantTag)

        html = widget.render("tags", None, attrs={"id": "alpha"})
        params = self.get_js_init_params(html)

        self.assertEqual(
            params,
            [
                "alpha",
                "/admin/tag-autocomplete/tests/restauranttag/",
                {"allowSpaces": True, "tagLimit": None, "autocompleteOnly": True},
            ],
        )

    def test_render_with_free_tagging_false(self):
        """Checks that free_tagging=False is passed to the inline script"""
        widget = widgets.AdminTagWidget(free_tagging=False)

        html = widget.render("tags", None, attrs={"id": "alpha"})
        params = self.get_js_init_params(html)

        self.assertEqual(
            params,
            [
                "alpha",
                "/admin/tag-autocomplete/",
                {"allowSpaces": True, "tagLimit": None, "autocompleteOnly": True},
            ],
        )

    def test_render_with_free_tagging_true(self):
        """free_tagging=True on the widget can also override the tag model setting free_tagging=False"""
        widget = widgets.AdminTagWidget(tag_model=RestaurantTag, free_tagging=True)

        html = widget.render("tags", None, attrs={"id": "alpha"})
        params = self.get_js_init_params(html)

        self.assertEqual(
            params,
            [
                "alpha",
                "/admin/tag-autocomplete/tests/restauranttag/",
                {"allowSpaces": True, "tagLimit": None, "autocompleteOnly": False},
            ],
        )

    @override_settings(WAGTAIL_TAG_SPACES_ALLOWED=True)
    def test_tags_help_text_spaces_allowed(self):
        """Checks that the tags help text html element content is correct when WAGTAIL_TAG_SPACES_ALLOWED is True"""
        widget = widgets.AdminTagWidget()
        help_text = widget.get_context(None, None, {})["widget"]["help_text"]

        html = widget.render("tags", None, {})

        self.assertEqual(
            help_text,
            'Multi-word tags with spaces will automatically be enclosed in double quotes (").',
        )

        self.assertIn(
            """<p class="help">%s</p>""" % escape(help_text),
            html,
        )

    @override_settings(WAGTAIL_TAG_SPACES_ALLOWED=False)
    def test_tags_help_text_no_spaces_allowed(self):
        """Checks that the tags help text html element content is correct when WAGTAIL_TAG_SPACES_ALLOWED is False"""
        widget = widgets.AdminTagWidget()
        help_text = widget.get_context(None, None, {})["widget"]["help_text"]

        html = widget.render("tags", None, {})

        self.assertEqual(
            help_text, "Tags can only consist of a single word, no spaces allowed."
        )

        self.assertIn(
            """<p class="help">%s</p>""" % escape(help_text),
            html,
        )

    @override_settings(TAG_LIMIT=3)
    def test_legacy_tag_limit_setting(self):
        widget = widgets.AdminTagWidget()
        with self.assertWarnsMessage(
            RemovedInWagtail80Warning,
            "The setting 'TAG_LIMIT' is deprecated. "
            "Please use 'WAGTAIL_TAG_LIMIT' instead.",
        ):
            html = widget.render("tags", None, attrs={"id": "alpha"})
        params = self.get_js_init_params(html)
        self.assertEqual(
            params,
            [
                "alpha",
                "/admin/tag-autocomplete/",
                {"allowSpaces": True, "tagLimit": 3, "autocompleteOnly": False},
            ],
        )

    @override_settings(TAG_SPACES_ALLOWED=False)
    def test_legacy_tag_spaces_allowed_setting(self):
        widget = widgets.AdminTagWidget()
        with self.assertWarnsMessage(
            RemovedInWagtail80Warning,
            "The setting 'TAG_SPACES_ALLOWED' is deprecated. "
            "Please use 'WAGTAIL_TAG_SPACES_ALLOWED' instead.",
        ):
            html = widget.render("tags", None, attrs={"id": "alpha"})
        params = self.get_js_init_params(html)
        self.assertEqual(
            params,
            [
                "alpha",
                "/admin/tag-autocomplete/",
                {"allowSpaces": False, "tagLimit": None, "autocompleteOnly": False},
            ],
        )


class TestTagField(TestCase):
    def setUp(self):
        RestaurantTag.objects.create(name="Italian", slug="italian")
        RestaurantTag.objects.create(name="Indian", slug="indian")

    def test_tag_whitelisting(self):
        class RestaurantTagForm(forms.Form):
            # RestaurantTag sets free_tagging=False at the model level
            tags = TagField(tag_model=RestaurantTag)

        form = RestaurantTagForm({"tags": "Italian, delicious"})
        self.assertTrue(form.is_valid())
        self.assertEqual(form.cleaned_data["tags"], ["Italian"])

    def test_override_free_tagging(self):
        class RestaurantTagForm(forms.Form):
            tags = TagField(tag_model=RestaurantTag, free_tagging=True)

        form = RestaurantTagForm({"tags": "Italian, delicious"})
        self.assertTrue(form.is_valid())
        self.assertEqual(set(form.cleaned_data["tags"]), {"Italian", "delicious"})

    def test_tag_over_one_hundred_characters(self):
        class RestaurantTagForm(forms.Form):
            tags = TagField(tag_model=RestaurantTag)

        tag_name = ""
        for _ in range(101):
            tag_name += "a"
        form = RestaurantTagForm({"tags": tag_name})
        self.assertFalse(form.is_valid())


class TestFilteredSelect(TestCase):
    def test_render(self):
        widget = widgets.FilteredSelect(
            choices=[
                (None, "----"),
                ("FR", "France", ["EU"]),
                ("JP", "Japan", ["AS"]),
                ("RU", "Russia", ["AS", "EU"]),
            ],
            filter_field="id_continent",
        )

        html = widget.render("country", "JP")
        self.assertHTMLEqual(
            html,
            """
            <select name="country" data-widget="filtered-select" data-filter-field="id_continent">
                <option value="">----</option>
                <option value="FR" data-filter-value="EU">France</option>
                <option value="JP" selected data-filter-value="AS">Japan</option>
                <option value="RU" data-filter-value="AS,EU">Russia</option>
            </select>
        """,
        )

    def test_optgroups(self):
        widget = widgets.FilteredSelect(
            choices=[
                (None, "----"),
                (
                    "Big countries",
                    [
                        ("FR", "France", ["EU"]),
                        ("JP", "Japan", ["AS"]),
                        ("RU", "Russia", ["AS", "EU"]),
                        ("MOON", "The moon"),
                    ],
                ),
                (
                    "Small countries",
                    [
                        ("AZ", "Azerbaijan", ["AS"]),
                        ("LI", "Liechtenstein", ["EU"]),
                    ],
                ),
                ("SK", "Slovakia", ["EU"]),
            ],
            filter_field="id_continent",
        )

        html = widget.render("country", "JP")
        self.assertHTMLEqual(
            html,
            """
            <select name="country" data-widget="filtered-select" data-filter-field="id_continent">
                <option value="">----</option>
                <optgroup label="Big countries">
                    <option value="FR" data-filter-value="EU">France</option>
                    <option value="JP" selected data-filter-value="AS">Japan</option>
                    <option value="RU" data-filter-value="AS,EU">Russia</option>
                    <option value="MOON">The moon</option>
                </optgroup>
                <optgroup label="Small countries">
                    <option value="AZ" data-filter-value="AS">Azerbaijan</option>
                    <option value="LI" data-filter-value="EU">Liechtenstein</option>
                </optgroup>
                <option value="SK" data-filter-value="EU">Slovakia</option>
            </select>
        """,
        )


class TestSlugInput(TestCase):
    def getAttrs(self, *args, **kwargs):
        return (
            BeautifulSoup(
                widgets.SlugInput(*args, **kwargs).render("slug", None),
                "html.parser",
            )
            .find("input")
            .attrs
        )

    def test_has_data_attr(self):
        widget = widgets.SlugInput()

        html = widget.render("test", None, attrs={"id": "test-id"})

        self.assertInHTML(
            '<input type="text" name="test" data-controller="w-slug" data-action="blur-&gt;w-slug#slugify w-sync:check-&gt;w-slug#compare w-sync:apply-&gt;w-slug#urlify:prevent" data-w-slug-allow-unicode-value data-w-slug-compare-as-param="urlify" data-w-slug-trim-value="true" id="test-id">',
            html,
        )

    @override_settings(WAGTAIL_ALLOW_UNICODE_SLUGS=False)
    def test_render_data_attrs_from_settings(self):
        widget = widgets.SlugInput()

        html = widget.render("test", None, attrs={"id": "test-id"})

        self.assertNotIn("data-w-slug-allow-unicode-value", html)

    def test_with_locale_and_formatters_not_provided(self):
        self.assertEqual(
            self.getAttrs(),
            {
                "data-action": "blur->w-slug#slugify w-sync:check->w-slug#compare w-sync:apply->w-slug#urlify:prevent",
                "data-controller": "w-slug",
                "data-w-slug-allow-unicode-value": "",
                "data-w-slug-compare-as-param": "urlify",
                # "data-w-slug-formatters-value":... # not included at all
                # "data-w-slug-locale-value":... # not included at all
                "data-w-slug-trim-value": "true",
                "name": "slug",
                "type": "text",
            },
        )

        self.assertEqual(
            self.getAttrs(formatters=[], locale=None),
            {
                "data-action": "blur->w-slug#slugify w-sync:check->w-slug#compare w-sync:apply->w-slug#urlify:prevent",
                "data-controller": "w-slug",
                "data-w-slug-allow-unicode-value": "",
                "data-w-slug-compare-as-param": "urlify",
                # "data-w-slug-formatters-value":... # not included at all
                # "data-w-slug-locale-value":... # not included at all
                "data-w-slug-trim-value": "true",
                "name": "slug",
                "type": "text",
            },
        )

    # Test formatters argument

    def test_with_formatters_provided_are_escaped(self):
        self.assertEqual(
            self.getAttrs(formatters=[(r"\D\s[']+", "'?'")])[
                "data-w-slug-formatters-value"
            ],
            '[[["\\\\D\\\\s[\']+","gu"],"\'?\'"]]',
        )

    def test_with_formatters_as_string(self):
        self.assertEqual(
            self.getAttrs(formatters=[r"\d"])["data-w-slug-formatters-value"],
            '[[["\\\\d","gu"],""]]',
        )

        # handling of inline flags
        self.assertEqual(
            self.getAttrs(formatters=[r"(?m)^\d+"])["data-w-slug-formatters-value"],
            '[[["^\\\\d+","gmu"],""]]',
        )

    def test_with_formatters_as_pattern(self):
        self.assertEqual(
            self.getAttrs(formatters=[re.compile(r"\d")])[
                "data-w-slug-formatters-value"
            ],
            '[[["\\\\d","gu"],""]]',
        )

        # handling of inline flags
        self.assertEqual(
            self.getAttrs(
                formatters=[re.compile(r"(?i)\b(?:and\|or\|the\|in\|of\|to)\b")]
            )["data-w-slug-formatters-value"],
            '[[["\\\\b(?:and\\\\|or\\\\|the\\\\|in\\\\|of\\\\|to)\\\\b","giu"],""]]',
        )

    def test_with_formatters_as_list_like_with_string(self):
        self.assertEqual(
            self.getAttrs(formatters=[["ABC"]])["data-w-slug-formatters-value"],
            '[[["ABC","gu"],""]]',
        )

        self.assertEqual(
            self.getAttrs(formatters=[(r"\d",)])["data-w-slug-formatters-value"],
            '[[["\\\\d","gu"],""]]',
        )

    def test_with_formatters_as_list_like_with_pattern(self):
        self.assertEqual(
            self.getAttrs(formatters=[[re.compile(r"(?i)!")]])[
                "data-w-slug-formatters-value"
            ],
            '[[["!","giu"],""]]',
        )
        self.assertEqual(
            self.getAttrs(formatters=[(re.compile("(?m)^A"),)])[
                "data-w-slug-formatters-value"
            ],
            '[[["^A","gmu"],""]]',
        )

    def test_with_formatters_with_replace(self):
        self.assertEqual(
            self.getAttrs(formatters=[(r"\d", "X")])["data-w-slug-formatters-value"],
            '[[["\\\\d","gu"],"X"]]',
        )

        self.assertEqual(
            self.getAttrs(formatters=[[re.compile(r"\d{1,3}"), "X"]])[
                "data-w-slug-formatters-value"
            ],
            '[[["\\\\d{1,3}","gu"],"X"]]',
        )

    def test_with_formatters_with_replace_and_flags(self):
        self.assertEqual(
            self.getAttrs(
                formatters=[
                    [re.compile(r"^(?!blog[-\s])", flags=re.MULTILINE), "blog-", "u"]
                ]
            )["data-w-slug-formatters-value"],
            '[[["^(?!blog[-\\\\s])","mu"],"blog-"]]',
        )

        self.assertEqual(
            self.getAttrs(
                formatters=[[re.compile(r"(?i)a*"), "Z", "g"], [r"the", "", "u"]]
            )["data-w-slug-formatters-value"],
            '[[["a*","gi"],"Z"],[["the","u"],""]]',
        )

    def test_with_multiple_formatters(self):
        self.assertEqual(
            self.getAttrs(
                formatters=[
                    r"\d",
                    [r"(?<!\S)Й", "Y"],
                    [re.compile(r"(?<!\S)Є"), "Ye", "u"],
                    (r"(?i)the",),
                ]
            )["data-w-slug-formatters-value"],
            '[[["\\\\d","gu"],""],[["(?<!\\\\S)\\u0419","gu"],"Y"],[["(?<!\\\\S)\\u0404","u"],"Ye"],[["the","giu"],""]]',
        )

    # Test locale argument

    def test_with_locale_provided(self):
        self.assertEqual(
            self.getAttrs(locale="uk-UK"),
            {
                "data-action": "blur->w-slug#slugify w-sync:check->w-slug#compare w-sync:apply->w-slug#urlify:prevent",
                "data-controller": "w-slug",
                "data-w-slug-allow-unicode-value": "",
                "data-w-slug-compare-as-param": "urlify",
                "data-w-slug-locale-value": "uk-UK",  # from provided locale
                "data-w-slug-trim-value": "true",
                "name": "slug",
                "type": "text",
            },
        )

        french = Locale.objects.create(language_code="fr")
        self.assertEqual(
            self.getAttrs(locale=french),
            {
                "data-action": "blur->w-slug#slugify w-sync:check->w-slug#compare w-sync:apply->w-slug#urlify:prevent",
                "data-controller": "w-slug",
                "data-w-slug-allow-unicode-value": "",
                "data-w-slug-compare-as-param": "urlify",
                "data-w-slug-locale-value": "fr",  # from provided locale
                "data-w-slug-trim-value": "true",
                "name": "slug",
                "type": "text",
            },
        )

        self.assertEqual(
            self.getAttrs(locale=False),
            {
                "data-action": "blur->w-slug#slugify w-sync:check->w-slug#compare w-sync:apply->w-slug#urlify:prevent",
                "data-controller": "w-slug",
                "data-w-slug-allow-unicode-value": "",
                "data-w-slug-compare-as-param": "urlify",
                "data-w-slug-locale-value": "und",  # from False (aka 'undetermined')
                "data-w-slug-trim-value": "true",
                "name": "slug",
                "type": "text",
            },
        )

    def test_with_locale_blank_override(self):
        self.assertEqual(
            self.getAttrs(locale=False)["data-w-slug-locale-value"],
            "und",
        )

        self.assertEqual(
            self.getAttrs(locale="")["data-w-slug-locale-value"],
            "und",
        )



import datetime
from unittest.mock import patch

from django.core.exceptions import ValidationError
from django.test import TestCase

from wagtail.fields import NoFutureDateValidator


class TestNoFutureDateValidator(TestCase):
    def setUp(self):
        self.validator = NoFutureDateValidator()

    def test_validates_past_date(self):
        """Past dates should pass validation"""
        past_date = datetime.date.today() - datetime.timedelta(days=1)
        # Should not raise ValidationError
        self.validator(past_date)

    def test_validates_today(self):
        """Today's date should pass validation"""
        today = datetime.date.today()
        # Should not raise ValidationError
        self.validator(today)

    def test_rejects_future_date(self):
        """Future dates should raise ValidationError"""
        future_date = datetime.date.today() + datetime.timedelta(days=1)
        with self.assertRaises(ValidationError) as cm:
            self.validator(future_date)

        self.assertEqual(cm.exception.code, "future_date")
        self.assertEqual(str(cm.exception.message), "Date cannot be in the future.")

    def test_validates_none_value(self):
        """None values should pass validation (let required validation handle empty values)"""
        # Should not raise ValidationError
        self.validator(None)

    def test_custom_message(self):
        """Test custom error message"""
        custom_message = "Custom future date error message"
        validator = NoFutureDateValidator(message=custom_message)
        future_date = datetime.date.today() + datetime.timedelta(days=1)

        with self.assertRaises(ValidationError) as cm:
            validator(future_date)

        self.assertEqual(str(cm.exception.message), custom_message)

    @patch("wagtail.fields.datetime")
    def test_validates_with_mocked_today(self, mock_datetime):
        """Test that validation uses the correct 'today' reference"""
        # Mock today to be 2024-01-15
        mock_today = datetime.date(2024, 1, 15)
        mock_datetime.date.today.return_value = mock_today

        # Test date exactly one day in the future
        future_date = datetime.date(2024, 1, 16)
        with self.assertRaises(ValidationError):
            self.validator(future_date)

        # Test date exactly 'today'
        self.validator(mock_today)  # Should not raise

        # Test past date
        past_date = datetime.date(2024, 1, 14)
        self.validator(past_date)  # Should not raise



# Generated by Django 5.2.3 on 2025-07-23 16:12

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("wagtailusers", "0014_userprofile_contrast"),
    ]

    operations = [
        migrations.AddField(
            model_name="userprofile",
            name="keyboard_shortcuts",
            field=models.BooleanField(
                default=True,
                help_text="Enable custom keyboard shortcuts specific to Wagtail.",
                verbose_name="Keyboard shortcuts",
            ),
        ),
    ]



from django import forms
from django.core.signing import BadSignature, Signer
from django.utils.translation import gettext_lazy as _

from wagtail.admin.forms.models import WagtailAdminModelForm
from wagtail.admin.widgets import AdminPageChooser
from wagtail.contrib.redirects.models import Redirect
from wagtail.models import Site


class RedirectForm(WagtailAdminModelForm):
    site = forms.ModelChoiceField(
        label=_("From site"),
        queryset=Site.objects.all(),
        required=False,
        empty_label=_("All sites"),
    )

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.fields["redirect_page"].widget = AdminPageChooser()

    required_css_class = "required"

    def clean(self):
        """
        The unique_together condition on the model is ignored if site is None, so need to
        check for duplicates manually
        """
        cleaned_data = super().clean()

        if cleaned_data.get("site") is None:
            old_path = cleaned_data.get("old_path")
            if old_path is None:
                # cleaned_data['old_path'] is empty because it has already failed validation,
                # so don't bother with our duplicate test
                return

            old_path = Redirect.normalise_path(old_path)
            duplicates = Redirect.objects.filter(old_path=old_path, site__isnull=True)
            if self.instance.pk:
                duplicates = duplicates.exclude(id=self.instance.pk)

            if duplicates:
                raise forms.ValidationError(
                    _("A redirect with this path already exists.")
                )

    class Meta(WagtailAdminModelForm.Meta):
        model = Redirect
        fields = ("old_path", "site", "is_permanent", "redirect_page", "redirect_link")


class ImportForm(forms.Form):
    import_file = forms.FileField(
        label=_("File to import"),
    )

    def __init__(self, allowed_extensions, *args, **kwargs):
        super().__init__(*args, **kwargs)

        accept = ",".join([f".{x}" for x in allowed_extensions])
        self.fields["import_file"].widget = forms.FileInput(attrs={"accept": accept})

        uppercased_extensions = [x.upper() for x in allowed_extensions]
        allowed_extensions_text = ", ".join(uppercased_extensions)
        help_text = _("Supported formats: %(supported_formats)s.") % {
            "supported_formats": allowed_extensions_text,
        }
        self.fields["import_file"].help_text = help_text


class ConfirmImportManagementForm(forms.Form):
    """
    Store the import file name and input format in the form so that it can be used in the next step

    The initial values are signed, to prevent them from being tampered with.
    """

    import_file_name = forms.CharField(widget=forms.HiddenInput())
    input_format = forms.CharField(widget=forms.HiddenInput())

    def __init__(self, *args, **kwargs):
        self.signer = Signer()
        initial = kwargs.get("initial", {})
        for key in {"import_file_name", "input_format"}:
            if key in initial:
                # Sign initial data so it cannot be tampered with
                initial[key] = self.signer.sign(initial[key])
        super().__init__(*args, **kwargs)

    def clean(self):
        cleaned_data = super().clean()
        for key in {"import_file_name", "input_format"}:
            try:
                cleaned_data[key] = self.signer.unsign(cleaned_data[key])
            except BadSignature as e:
                raise forms.ValidationError(e.message)
        return cleaned_data


class ConfirmImportForm(ConfirmImportManagementForm):
    from_index = forms.ChoiceField(
        label=_("From field"),
        choices=(),
    )
    to_index = forms.ChoiceField(
        label=_("To field"),
        choices=(),
    )
    site = forms.ModelChoiceField(
        label=_("From site"),
        queryset=Site.objects.all(),
        required=False,
        empty_label=_("All sites"),
    )
    permanent = forms.BooleanField(initial=True, required=False)

    def __init__(self, headers, *args, **kwargs):
        super().__init__(*args, **kwargs)

        choices = []
        for i, f in enumerate(headers):
            choices.append([str(i), f])
        if len(headers) > 1:
            choices.insert(0, ("", "---"))

        self.fields["from_index"].choices = choices
        self.fields["to_index"].choices = choices



from django.db import connection, migrations

# This migration takes on the base model defined in 0005_create_indexentry and adds certain fields that are specific to each database system
class Migration(migrations.Migration):

    dependencies = [
        ("wagtailsearch", "0008_remove_query_and_querydailyhits_models"),
    ]

    # Reverse the usage of ngram parser with MySQL introduced in
    # ``0006_customise_indexentry.py``.
    #
    # Ngram parser is designed primarily for languages which to not use spaces to
    # delimit words (most commonly CJK languages). When using ngram with English or
    # other non-CJK languages, it has the adverse affect of parsing words into
    # non-sensical queries that return no results. MySQL default ngram token size is 2
    # characters, so for example the word "wagtail" would be parsed into 6 separate
    # words as follows":
    #
    #    "wa" "ag" "gt" "ta" "ai" "il"
    #
    # In English, these tokens would provide useless search results, and in most cases,
    # no results are returned at all when using autocomplete from the Wagtail admin.
    #
    # See: https://dev.mysql.com/doc/refman/8.0/en/fulltext-search-ngram.html
    if connection.vendor == "mysql" and not connection.mysql_is_mariadb:
        operations = [
            migrations.RunSQL(
                sql="""
                ALTER TABLE wagtailsearch_indexentry
                    DROP INDEX `fulltext_autocomplete`;

                ALTER TABLE wagtailsearch_indexentry
                    ADD FULLTEXT INDEX `fulltext_autocomplete` (`autocomplete`);
                """,
                reverse_sql="""
                ALTER TABLE wagtailsearch_indexentry
                    DROP INDEX `fulltext_autocomplete`;

                ALTER TABLE wagtailsearch_indexentry
                    ADD FULLTEXT INDEX `fulltext_autocomplete` (`autocomplete`)
                    WITH PARSER ngram;
                """,
            )
        ]



# Generated by Django 4.1.4 on 2022-12-23 09:32

from django.db import migrations, models
import taggit.managers
import wagtail.search.index


class Migration(migrations.Migration):

    dependencies = [
        ("taggit", "0001_initial"),
        ("searchtests", "0001_initial"),
    ]

    operations = [
        migrations.CreateModel(
            name="UnindexedBook",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("title", models.CharField(max_length=255)),
                ("publication_date", models.DateField()),
                ("number_of_pages", models.IntegerField()),
                (
                    "tags",
                    taggit.managers.TaggableManager(
                        help_text="A comma-separated list of tags.",
                        through="taggit.TaggedItem",
                        to="taggit.Tag",
                        verbose_name="Tags",
                    ),
                ),
            ],
            bases=(wagtail.search.index.Indexed, models.Model),
        ),
    ]



from django.core.exceptions import ValidationError
from django.test import TestCase, override_settings

from wagtail.coreutils import get_dummy_request
from wagtail.models import Page, Site


class TestSiteNaturalKey(TestCase):
    def test_natural_key(self):
        site = Site(hostname="example.com", port=8080)
        self.assertEqual(site.natural_key(), ("example.com", 8080))

    def test_get_by_natural_key(self):
        site = Site.objects.create(
            hostname="example.com", port=8080, root_page=Page.objects.get(pk=2)
        )
        self.assertEqual(Site.objects.get_by_natural_key("example.com", 8080), site)


class TestSiteUrl(TestCase):
    def test_root_url_http(self):
        site = Site(hostname="example.com", port=80)
        self.assertEqual(site.root_url, "http://example.com")

    def test_root_url_https(self):
        site = Site(hostname="example.com", port=443)
        self.assertEqual(site.root_url, "https://example.com")

    def test_root_url_custom_port(self):
        site = Site(hostname="example.com", port=8000)
        self.assertEqual(site.root_url, "http://example.com:8000")


class TestSiteNameDisplay(TestCase):
    def test_site_name_not_default(self):
        site = Site(
            hostname="example.com",
            port=80,
            site_name="example dot com",
            is_default_site=False,
        )
        self.assertEqual(site.__str__(), "example dot com")

    def test_site_name_default(self):
        site = Site(
            hostname="example.com",
            port=80,
            site_name="example dot com",
            is_default_site=True,
        )
        self.assertEqual(site.__str__(), "example dot com [default]")

    def test_no_site_name_not_default_port_80(self):
        site = Site(hostname="example.com", port=80, is_default_site=False)
        self.assertEqual(site.__str__(), "example.com")

    def test_no_site_name_default_port_80(self):
        site = Site(hostname="example.com", port=80, is_default_site=True)
        self.assertEqual(site.__str__(), "example.com [default]")

    def test_no_site_name_not_default_port_n(self):
        site = Site(hostname="example.com", port=8080, is_default_site=False)
        self.assertEqual(site.__str__(), "example.com:8080")

    def test_no_site_name_default_port_n(self):
        site = Site(hostname="example.com", port=8080, is_default_site=True)
        self.assertEqual(site.__str__(), "example.com:8080 [default]")


class TestSiteOrdering(TestCase):
    def setUp(self):
        self.root_page = Page.objects.get(pk=2)
        Site.objects.all().delete()  # Drop the initial site.

    def test_site_order_by_hostname(self):
        site_1 = Site.objects.create(hostname="charly.com", root_page=self.root_page)
        site_2 = Site.objects.create(hostname="bravo.com", root_page=self.root_page)
        site_3 = Site.objects.create(hostname="alfa.com", root_page=self.root_page)
        self.assertEqual(
            list(Site.objects.all().values_list("id", flat=True)),
            [site_3.id, site_2.id, site_1.id],
        )

    def test_site_order_by_hostname_upper(self):
        site_1 = Site.objects.create(hostname="charly.com", root_page=self.root_page)
        site_2 = Site.objects.create(hostname="Bravo.com", root_page=self.root_page)
        site_3 = Site.objects.create(hostname="alfa.com", root_page=self.root_page)
        self.assertEqual(
            list(Site.objects.all().values_list("id", flat=True)),
            [site_3.id, site_2.id, site_1.id],
        )

    def test_site_order_by_hostname_site_name_irrelevant(self):
        site_1 = Site.objects.create(
            hostname="charly.com", site_name="X-ray", root_page=self.root_page
        )
        site_2 = Site.objects.create(
            hostname="bravo.com", site_name="Yankee", root_page=self.root_page
        )
        site_3 = Site.objects.create(
            hostname="alfa.com", site_name="Zulu", root_page=self.root_page
        )
        self.assertEqual(
            list(Site.objects.all().values_list("id", flat=True)),
            [site_3.id, site_2.id, site_1.id],
        )


@override_settings(ALLOWED_HOSTS=["example.com", "unknown.com", "127.0.0.1", "[::1]"])
class TestFindSiteForRequest(TestCase):
    def setUp(self):
        self.default_site = Site.objects.get()
        self.site = Site.objects.create(
            hostname="example.com", port=80, root_page=Page.objects.get(pk=2)
        )

    def test_dummy_request(self):
        request = get_dummy_request(site=self.site)
        self.assertEqual(Site.find_for_request(request), self.site)

    def test_with_host(self):
        request = get_dummy_request()
        request.META.update({"HTTP_HOST": "example.com", "SERVER_PORT": 80})
        self.assertEqual(Site.find_for_request(request), self.site)

    def test_with_unknown_host(self):
        request = get_dummy_request()
        request.META.update({"HTTP_HOST": "unknown.com", "SERVER_PORT": 80})
        self.assertEqual(Site.find_for_request(request), self.default_site)

    def test_with_server_name(self):
        request = get_dummy_request()
        request.META.update({"SERVER_NAME": "example.com", "SERVER_PORT": 80})
        self.assertEqual(Site.find_for_request(request), self.site)

    def test_with_x_forwarded_host(self):
        with self.settings(USE_X_FORWARDED_HOST=True):
            request = get_dummy_request()
            request.META.update(
                {"HTTP_X_FORWARDED_HOST": "example.com", "SERVER_PORT": 80}
            )
            self.assertEqual(Site.find_for_request(request), self.site)

    def test_ipv4_host(self):
        request = get_dummy_request()
        request.META.update({"SERVER_NAME": "127.0.0.1", "SERVER_PORT": 80})
        self.assertEqual(Site.find_for_request(request), self.default_site)

    def test_ipv6_host(self):
        request = get_dummy_request()
        request.META.update({"SERVER_NAME": "[::1]", "SERVER_PORT": 80})
        self.assertEqual(Site.find_for_request(request), self.default_site)


class TestDefaultSite(TestCase):
    def test_create_default_site(self):
        Site.objects.all().delete()
        Site.objects.create(
            hostname="test.com", is_default_site=True, root_page=Page.objects.get(pk=2)
        )
        self.assertTrue(Site.objects.filter(is_default_site=True).exists())

    def test_change_default_site(self):
        default = Site.objects.get(is_default_site=True)
        default.is_default_site = False
        default.save()

        Site.objects.create(
            hostname="test.com", is_default_site=True, root_page=Page.objects.get(pk=2)
        )
        self.assertTrue(Site.objects.filter(is_default_site=True).exists())

    def test_there_can_only_be_one(self):
        site = Site(
            hostname="test.com", is_default_site=True, root_page=Page.objects.get(pk=2)
        )
        with self.assertRaises(ValidationError):
            site.clean_fields()

    def test_oops_there_is_more_than_one(self):
        Site.objects.create(
            hostname="example.com",
            is_default_site=True,
            root_page=Page.objects.get(pk=2),
        )

        site = Site(
            hostname="test.com", is_default_site=True, root_page=Page.objects.get(pk=2)
        )
        with self.assertRaises(Site.MultipleObjectsReturned):
            # If there already are multiple default sites, you're in trouble
            site.clean_fields()


class TestGetSiteRootPaths(TestCase):
    def setUp(self):
        self.default_site = Site.objects.get()
        self.abc_site = Site.objects.create(
            hostname="abc.com", root_page=self.default_site.root_page
        )
        self.def_site = Site.objects.create(
            hostname="def.com", root_page=self.default_site.root_page
        )

        # Changing the hostname to show that being the default site takes
        # promotes a site over the alphabetical ordering of hostname
        self.default_site.hostname = "xyz.com"
        self.default_site.save()

    def test_result_order_when_multiple_sites_share_the_same_root_page(self):
        result = Site.get_site_root_paths()

        # An entry for the default site should come first
        self.assertEqual(result[0][0], self.default_site.id)

        # Followed by entries for others in 'host' alphabetical order
        self.assertEqual(result[1][0], self.abc_site.id)
        self.assertEqual(result[2][0], self.def_site.id)



import warnings

from wagtail.admin.telepath.widgets import (  # noqa: F401
    CheckboxInputAdapter,
    RadioSelectAdapter,
    SelectAdapter,
    ValidationErrorAdapter,
    WidgetAdapter,
)
from wagtail.utils.deprecation import RemovedInWagtail80Warning

warnings.warn(
    "wagtail.widget_adapters has been moved to wagtail.admin.telepath.widgets",
    RemovedInWagtail80Warning,
    stacklevel=2,
)



from django.conf import settings
from django.contrib.auth import get_user_model
from django.contrib.auth.models import Group, Permission
from django.test import TestCase
from django.urls import reverse
from freezegun import freeze_time

from wagtail.admin.staticfiles import versioned_static
from wagtail.models import Page
from wagtail.test.testapp.models import (
    DefaultStreamPage,
    EventPage,
    FormClassAdditionalFieldPage,
    SecretPage,
)
from wagtail.test.utils import WagtailTestUtils
from wagtail.test.utils.template_tests import AdminTemplateTestUtils
from wagtail.test.utils.timestamps import local_datetime


class TestRevisions(WagtailTestUtils, TestCase):
    fixtures = ["test.json"]

    def setUp(self):
        self.christmas_event = EventPage.objects.get(url_path="/home/events/christmas/")
        self.christmas_event.title = "Last Christmas"
        self.christmas_event.date_from = "2013-12-25"
        self.christmas_event.body = (
            "<p>Last Christmas I gave you my heart, "
            "but the very next day you gave it away</p>"
        )
        self.last_christmas_revision = self.christmas_event.save_revision()
        self.last_christmas_revision.created_at = local_datetime(2013, 12, 25)
        self.last_christmas_revision.save()

        self.christmas_event.title = "This Christmas"
        self.christmas_event.date_from = "2014-12-25"
        self.christmas_event.body = (
            "<p>This year, to save me from tears, I'll give it to someone special</p>"
        )
        self.this_christmas_revision = self.christmas_event.save_revision()
        self.this_christmas_revision.created_at = local_datetime(2014, 12, 25)
        self.this_christmas_revision.save()

        self.login()

    def test_get_revisions_index(self):
        response = self.client.get(
            reverse(
                "wagtailadmin_pages:revisions_index", args=(self.christmas_event.id,)
            )
        )
        history_url = reverse(
            "wagtailadmin_pages:history", args=(self.christmas_event.id,)
        )
        self.assertRedirects(response, history_url)

    def request_preview_revision(self):
        last_christmas_preview_url = reverse(
            "wagtailadmin_pages:revisions_view",
            args=(self.christmas_event.id, self.last_christmas_revision.id),
        )
        return self.client.get(last_christmas_preview_url)

    def test_preview_revision(self):
        response = self.request_preview_revision()

        self.assertEqual(response.status_code, 200)
        self.assertContains(response, "Last Christmas I gave you my heart")

        # Should show edit link in the userbar
        # https://github.com/wagtail/wagtail/issues/10002
        self.assertContains(response, "Edit this page")
        self.assertContains(
            response,
            reverse("wagtailadmin_pages:edit", args=(self.christmas_event.id,)),
        )

    def test_preview_revision_with_no_page_permissions_redirects_to_admin(self):
        admin_only_user = self.create_user(
            username="admin_only", email="admin_only@email.com", password="password"
        )
        admin_only_user.user_permissions.add(
            Permission.objects.get_by_natural_key(
                codename="access_admin", app_label="wagtailadmin", model="admin"
            )
        )

        self.login(user=admin_only_user)
        response = self.request_preview_revision()

        self.assertEqual(response.status_code, 302)
        self.assertEqual(response["Location"], reverse("wagtailadmin_home"))

    def test_preview_revision_forbidden_without_permission(self):
        # Alter the editors group so it has no permissions for Christmas page.
        st_patricks = Page.objects.get(slug="saint-patrick")
        editors_group = Group.objects.get(name="Site-wide editors")
        editors_group.page_permissions.update(page_id=st_patricks.id)

        editor = get_user_model().objects.get(email="siteeditor@example.com")

        self.login(editor)
        response = self.request_preview_revision()

        self.assertEqual(response.status_code, 302)

    def test_revert_revision(self):
        last_christmas_preview_url = reverse(
            "wagtailadmin_pages:revisions_revert",
            args=(self.christmas_event.id, self.last_christmas_revision.id),
        )
        response = self.client.get(last_christmas_preview_url)
        self.assertEqual(response.status_code, 200)

        self.assertContains(response, "Editing Event page")
        self.assertContains(response, "You are viewing a previous version of this page")

        # Form should show the content of the revision, not the current draft
        self.assertContains(response, "Last Christmas I gave you my heart")

        # Form should use the revisions revert URL as the action
        soup = self.get_soup(response.content)
        form = soup.select_one("form[data-edit-form]")
        self.assertIsNotNone(form)
        self.assertEqual(
            form.get("action"),
            reverse(
                "wagtailadmin_pages:revisions_revert",
                args=(self.christmas_event.id, self.last_christmas_revision.id),
            ),
        )

        # Buttons should be relabelled
        self.assertContains(response, "Replace current draft")
        self.assertContains(response, "Publish this version")

    @freeze_time("2014-12-20 12:00:00")
    def test_scheduled_revision(self):
        if settings.USE_TZ:
            # 12:00 UTC
            self.christmas_event.go_live_at = "2014-12-26T12:00:00.000Z"
        else:
            # 12:00 in no specific timezone
            self.christmas_event.go_live_at = "2014-12-26T12:00:00"
        this_christmas_revision = self.christmas_event.save_revision(log_action=True)
        this_christmas_revision.publish(log_action=True)
        this_christmas_unschedule_url = reverse(
            "wagtailadmin_pages:revisions_unschedule",
            args=(self.christmas_event.id, this_christmas_revision.id),
        )
        response = self.client.get(
            reverse("wagtailadmin_pages:history", args=(self.christmas_event.id,))
        )
        self.assertEqual(response.status_code, 200)

        if settings.USE_TZ:
            # the default timezone is "Asia/Tokyo", so we expect UTC +9
            expected_date_string = "Dec. 26, 2014, 9 p.m."
        else:
            expected_date_string = "Dec. 26, 2014, noon"

        self.assertContains(
            response, f"Page scheduled for publishing at {expected_date_string}"
        )
        self.assertContains(response, this_christmas_unschedule_url)


class TestStreamRevisions(WagtailTestUtils, TestCase):
    def setUp(self):
        self.root_page = Page.objects.get(id=2)

        self.test_page = DefaultStreamPage(
            title="A DefaultStreamPage",
            slug="a-defaultstreampage",
        )
        self.root_page.add_child(instance=self.test_page)

        self.test_page.title = "An Updated DefaultStreamPage"
        self.first_revision = self.test_page.save_revision()
        self.first_revision.created_at = local_datetime(2022, 5, 10)
        self.first_revision.save()

        self.login()

    def test_revert_revision(self):
        test_page_revert_url = reverse(
            "wagtailadmin_pages:revisions_revert",
            args=(self.test_page.id, self.first_revision.id),
        )
        response = self.client.get(test_page_revert_url)
        self.assertEqual(response.status_code, 200)

        html = response.content.decode()
        blocks_js = versioned_static("wagtailadmin/js/telepath/blocks.js")
        streamfield_css = versioned_static("wagtailadmin/css/panels/streamfield.css")

        # The media files for StreamField should be included in the HTML
        self.assertTagInHTML(f'<script src="{blocks_js}"></script>', html)
        self.assertTagInHTML(
            f'<link href="{streamfield_css}" media="all" rel="stylesheet">',
            html,
            allow_extra_attrs=True,  # Django 4.1 removes the type="text/css" attribute
        )


class TestCompareRevisions(AdminTemplateTestUtils, WagtailTestUtils, TestCase):
    # Actual tests for the comparison classes can be found in test_compare.py

    base_breadcrumb_items = []
    fixtures = ["test.json"]

    def setUp(self):
        self.christmas_event = EventPage.objects.get(url_path="/home/events/christmas/")
        self.christmas_event.title = "Last Christmas"
        self.christmas_event.date_from = "2013-12-25"
        self.christmas_event.body = (
            "<p>Last Christmas I gave you my heart, "
            "but the very next day you gave it away</p>"
        )
        self.last_christmas_revision = self.christmas_event.save_revision()
        self.last_christmas_revision.created_at = local_datetime(2013, 12, 25)
        self.last_christmas_revision.save()

        self.christmas_event.title = "This Christmas"
        self.christmas_event.date_from = "2014-12-25"
        self.christmas_event.body = (
            "<p>This year, to save me from tears, I'll give it to someone special</p>"
        )
        self.this_christmas_revision = self.christmas_event.save_revision()
        self.this_christmas_revision.created_at = local_datetime(2014, 12, 25)
        self.this_christmas_revision.save()

        self.login()

    def test_compare_revisions(self):
        compare_url = reverse(
            "wagtailadmin_pages:revisions_compare",
            args=(
                self.christmas_event.id,
                self.last_christmas_revision.id,
                self.this_christmas_revision.id,
            ),
        )
        response = self.client.get(compare_url)
        self.assertEqual(response.status_code, 200)

        self.assertContains(
            response,
            '<span class="deletion">Last Christmas I gave you my heart, but the very next day you gave it away</span><span class="addition">This year, to save me from tears, I&#39;ll give it to someone special</span>',
            html=True,
        )

        history_url = reverse(
            "wagtailadmin_pages:history", args=(self.christmas_event.id,)
        )

        self.assertBreadcrumbsItemsRendered(
            [
                {
                    "url": reverse("wagtailadmin_explore_root")
                    if page.is_root()
                    else reverse("wagtailadmin_explore", args=(page.pk,)),
                    "label": page.get_admin_display_title(),
                }
                for page in self.christmas_event.get_ancestors(inclusive=True)
            ]
            + [
                {"url": history_url, "label": "History"},
                {"url": "", "label": "Compare", "sublabel": "This Christmas"},
            ],
            response.content,
        )

        soup = self.get_soup(response.content)
        edit_url = reverse("wagtailadmin_pages:edit", args=(self.christmas_event.id,))
        edit_button = soup.select_one(f"a.w-header-button[href='{edit_url}']")
        self.assertIsNotNone(edit_button)
        self.assertEqual(edit_button.text.strip(), "Edit")

    def test_compare_revisions_earliest(self):
        compare_url = reverse(
            "wagtailadmin_pages:revisions_compare",
            args=(self.christmas_event.id, "earliest", self.this_christmas_revision.id),
        )
        response = self.client.get(compare_url)
        self.assertEqual(response.status_code, 200)

        self.assertContains(
            response,
            '<span class="deletion">Last Christmas I gave you my heart, but the very next day you gave it away</span><span class="addition">This year, to save me from tears, I&#39;ll give it to someone special</span>',
            html=True,
        )

    def test_compare_revisions_latest(self):
        compare_url = reverse(
            "wagtailadmin_pages:revisions_compare",
            args=(self.christmas_event.id, self.last_christmas_revision.id, "latest"),
        )
        response = self.client.get(compare_url)
        self.assertEqual(response.status_code, 200)

        self.assertContains(
            response,
            '<span class="deletion">Last Christmas I gave you my heart, but the very next day you gave it away</span><span class="addition">This year, to save me from tears, I&#39;ll give it to someone special</span>',
            html=True,
        )

    def test_compare_revisions_live(self):
        # Mess with the live version, bypassing revisions
        self.christmas_event.body = (
            "<p>This year, to save me from tears, I'll just feed it to the dog</p>"
        )
        self.christmas_event.save(update_fields=["body"])

        compare_url = reverse(
            "wagtailadmin_pages:revisions_compare",
            args=(self.christmas_event.id, self.last_christmas_revision.id, "live"),
        )
        response = self.client.get(compare_url)
        self.assertEqual(response.status_code, 200)

        self.assertContains(
            response,
            '<span class="deletion">Last Christmas I gave you my heart, but the very next day you gave it away</span><span class="addition">This year, to save me from tears, I&#39;ll just feed it to the dog</span>',
            html=True,
        )


class TestCompareRevisionsWithPerUserEditHandlers(WagtailTestUtils, TestCase):
    fixtures = ["test.json"]

    def setUp(self):
        self.home = Page.objects.get(url_path="/home/")
        self.secret_page = SecretPage(
            title="Secret page",
            boring_data="InnocentCorp is the leading supplier of door hinges",
            secret_data="for flying saucers",
        )
        self.home.add_child(instance=self.secret_page)
        self.old_revision = self.secret_page.save_revision()
        self.secret_page.boring_data = (
            "InnocentCorp is the leading supplier of rubber sprockets"
        )
        self.secret_page.secret_data = "for fake moon landings"
        self.new_revision = self.secret_page.save_revision()
        self.compare_url = reverse(
            "wagtailadmin_pages:revisions_compare",
            args=(self.secret_page.id, self.old_revision.id, self.new_revision.id),
        )

    def test_comparison_as_superuser(self):
        self.login()
        response = self.client.get(self.compare_url)
        self.assertEqual(response.status_code, 200)

        self.assertContains(
            response,
            'InnocentCorp is the leading supplier of <span class="deletion">door hinges</span><span class="addition">rubber sprockets</span>',
            html=True,
        )
        self.assertContains(
            response,
            'for <span class="deletion">flying saucers</span><span class="addition">fake moon landings</span>',
            html=True,
        )

    def test_comparison_as_ordinary_user(self):
        user = self.create_user(username="editor", password="password")
        user.groups.add(Group.objects.get(name="Site-wide editors"))
        self.login(username="editor", password="password")

        response = self.client.get(self.compare_url)
        self.assertEqual(response.status_code, 200)

        self.assertContains(
            response,
            'InnocentCorp is the leading supplier of <span class="deletion">door hinges</span><span class="addition">rubber sprockets</span>',
            html=True,
        )
        self.assertNotContains(
            response,
            "moon landings",
        )


class TestCompareRevisionsWithNonModelField(WagtailTestUtils, TestCase):
    """
    Tests if form fields defined in the base_form_class will not be included.
    in revisions view as they are not actually on the model.
    Flagged in issue #3737
    Note: Actual tests for comparison classes can be found in test_compare.py
    """

    fixtures = ["test.json"]
    # FormClassAdditionalFieldPage

    def setUp(self):
        # Find root page
        self.root_page = Page.objects.get(id=2)

        # Add child page of class with base_form_class override
        # non model field is 'code'
        self.test_page = FormClassAdditionalFieldPage(
            title="A Statement",
            slug="a-statement",
            location="Early Morning Cafe, Mainland, NZ",
            body="<p>hello</p>",
        )
        self.root_page.add_child(instance=self.test_page)

        # add new revision
        self.test_page.title = "Statement"
        self.test_page.location = "Victory Monument, Bangkok"
        self.test_page.body = "<p>I would like very much to go into the forrest.</p>"
        self.test_page_revision = self.test_page.save_revision()
        self.test_page_revision.created_at = local_datetime(2017, 10, 15)
        self.test_page_revision.save()

        # add another new revision
        self.test_page.title = "True Statement"
        self.test_page.location = "Victory Monument, Bangkok"
        self.test_page.body = "<p>I would like very much to go into the forest.</p>"
        self.test_page_revision_new = self.test_page.save_revision()
        self.test_page_revision_new.created_at = local_datetime(2017, 10, 16)
        self.test_page_revision_new.save()

        self.login()

    def test_base_form_class_used(self):
        """First ensure that the non-model field is appearing in edit."""
        edit_url = reverse(
            "wagtailadmin_pages:add",
            args=("tests", "formclassadditionalfieldpage", self.test_page.id),
        )
        response = self.client.get(edit_url)
        self.assertContains(
            response,
            '<input type="text" name="code" aria-describedby="panel-child-content-child-code-helptext" required id="id_code" maxlength="5" />',
            html=True,
        )

    def test_compare_revisions(self):
        """Confirm that the non-model field is not shown in revision."""
        compare_url = reverse(
            "wagtailadmin_pages:revisions_compare",
            args=(
                self.test_page.id,
                self.test_page_revision.id,
                self.test_page_revision_new.id,
            ),
        )
        response = self.client.get(compare_url)
        self.assertContains(
            response,
            '<span class="deletion">forrest.</span><span class="addition">forest.</span>',
        )
        # should not contain the field defined in the formclass used
        self.assertNotContains(response, "<h2>Code:</h2>")


class TestRevisionsUnschedule(WagtailTestUtils, TestCase):
    fixtures = ["test.json"]

    def setUp(self):
        self.christmas_event = EventPage.objects.get(url_path="/home/events/christmas/")
        self.christmas_event.title = "Last Christmas"
        self.christmas_event.date_from = "2013-12-25"
        self.christmas_event.body = (
            "<p>Last Christmas I gave you my heart, "
            "but the very next day you gave it away</p>"
        )
        self.last_christmas_revision = self.christmas_event.save_revision()
        self.last_christmas_revision.created_at = local_datetime(2013, 12, 25)
        self.last_christmas_revision.save()
        self.last_christmas_revision.publish()

        self.christmas_event.title = "This Christmas"
        self.christmas_event.date_from = "2014-12-25"
        self.christmas_event.body = (
            "<p>This year, to save me from tears, I'll give it to someone special</p>"
        )
        self.this_christmas_revision = self.christmas_event.save_revision()
        self.this_christmas_revision.created_at = local_datetime(2014, 12, 24)
        self.this_christmas_revision.save()

        self.this_christmas_revision.approved_go_live_at = local_datetime(2014, 12, 25)
        self.this_christmas_revision.save()

        self.user = self.login()

    def test_unschedule_view(self):
        """
        This tests that the unschedule view responds with a confirm page
        """
        response = self.client.get(
            reverse(
                "wagtailadmin_pages:revisions_unschedule",
                args=(self.christmas_event.id, self.this_christmas_revision.id),
            )
        )
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(
            response, "wagtailadmin/shared/revisions/confirm_unschedule.html"
        )

    def test_unschedule_view_invalid_page_id(self):
        """
        This tests that the unschedule view returns an error if the page id is invalid
        """
        # Get unschedule page
        response = self.client.get(
            reverse("wagtailadmin_pages:revisions_unschedule", args=(12345, 67894))
        )

        # Check that the user received a 404 response
        self.assertEqual(response.status_code, 404)

    def test_unschedule_view_invalid_revision_id(self):
        """
        This tests that the unschedule view returns an error if the page id is invalid
        """
        # Get unschedule page
        response = self.client.get(
            reverse(
                "wagtailadmin_pages:revisions_unschedule",
                args=(self.christmas_event.id, 67894),
            )
        )

        # Check that the user received a 404 response
        self.assertEqual(response.status_code, 404)

    def test_unschedule_view_bad_permissions(self):
        """
        This tests that the unschedule view doesn't allow users without publish permissions
        """
        # Remove privileges from user
        self.user.is_superuser = False
        self.user.user_permissions.add(
            Permission.objects.get(
                content_type__app_label="wagtailadmin", codename="access_admin"
            )
        )
        self.user.save()

        # Get unschedule page
        response = self.client.get(
            reverse(
                "wagtailadmin_pages:revisions_unschedule",
                args=(self.christmas_event.id, self.this_christmas_revision.id),
            )
        )

        # Check that the user received a 302 redirected response
        self.assertEqual(response.status_code, 302)

    def test_unschedule_view_post(self):
        """
        This posts to the unschedule view and checks that the revision was unscheduled
        """

        # Post to the unschedule page
        response = self.client.post(
            reverse(
                "wagtailadmin_pages:revisions_unschedule",
                args=(self.christmas_event.id, self.this_christmas_revision.id),
            )
        )

        # Should be redirected to page history
        self.assertRedirects(
            response,
            reverse("wagtailadmin_pages:history", args=(self.christmas_event.id,)),
        )

        # Check that the page has no approved_schedule
        self.assertFalse(
            EventPage.objects.get(id=self.christmas_event.id).approved_schedule
        )

        # Check that the approved_go_live_at has been cleared from the revision
        self.assertIsNone(
            self.christmas_event.revisions.get(
                id=self.this_christmas_revision.id
            ).approved_go_live_at
        )

    def test_unschedule_view_post_with_next_url(self):
        """
        This tests that the redirect response follows the "next" parameter
        """

        unschedule_url = reverse(
            "wagtailadmin_pages:revisions_unschedule",
            args=(self.christmas_event.id, self.this_christmas_revision.id),
        )
        edit_url = reverse("wagtailadmin_pages:edit", args=(self.christmas_event.id,))

        # Post to the unschedule page
        response = self.client.post(f"{unschedule_url}?next={edit_url}")

        # Should be redirected to edit page
        self.assertRedirects(response, edit_url)

        # Check that the page has no approved_schedule
        self.assertFalse(
            EventPage.objects.get(id=self.christmas_event.id).approved_schedule
        )

        # Check that the approved_go_live_at has been cleared from the revision
        self.assertIsNone(
            self.christmas_event.revisions.get(
                id=self.this_christmas_revision.id
            ).approved_go_live_at
        )


class TestRevisionsUnscheduleForUnpublishedPages(WagtailTestUtils, TestCase):
    fixtures = ["test.json"]

    def setUp(self):
        self.unpublished_event = EventPage.objects.get(
            url_path="/home/events/tentative-unpublished-event/"
        )
        self.unpublished_event.title = "Unpublished Page"
        self.unpublished_event.date_from = "2014-12-25"
        self.unpublished_event.body = "<p>Some Content</p>"
        self.unpublished_revision = self.unpublished_event.save_revision()
        self.unpublished_revision.created_at = local_datetime(2014, 12, 25)
        self.unpublished_revision.save()

        self.user = self.login()

    def test_unschedule_view(self):
        """
        This tests that the unschedule view responds with a confirm page
        """
        response = self.client.get(
            reverse(
                "wagtailadmin_pages:revisions_unschedule",
                args=(self.unpublished_event.id, self.unpublished_revision.id),
            )
        )
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(
            response, "wagtailadmin/shared/revisions/confirm_unschedule.html"
        )

    def test_unschedule_view_post(self):
        """
        This posts to the unschedule view and checks that the revision was unscheduled
        """

        # Post to the unschedule page
        response = self.client.post(
            reverse(
                "wagtailadmin_pages:revisions_unschedule",
                args=(self.unpublished_event.id, self.unpublished_revision.id),
            )
        )

        # Should be redirected to page history
        self.assertRedirects(
            response,
            reverse("wagtailadmin_pages:history", args=(self.unpublished_event.id,)),
        )

        # Check that the page has no approved_schedule
        self.assertFalse(
            EventPage.objects.get(id=self.unpublished_event.id).approved_schedule
        )

        # Check that the approved_go_live_at has been cleared from the revision
        self.assertIsNone(
            self.unpublished_event.revisions.get(
                id=self.unpublished_revision.id
            ).approved_go_live_at
        )
